\name{PlainVanillaOptions}

\alias{PlainVanillaOptions}

\alias{GBSOption}
\alias{GBSCharacteristics}
\alias{BlackScholesOption}
\alias{GBSGreeks}
\alias{GBSVolatility}

\alias{Black76Option}
\alias{MiltersenSchwartzOption}

\alias{NDF}
\alias{CND}
\alias{CBND}

\alias{print.option}
\alias{summary.option}


\title{Valuation of Plain Vanilla Options}


\description{

	This is a collection of functions to valuate plain vanilla options.
	Included are functions for the Generalized Block-Scholes option 
	pricing model, for options on futures, approximative formulas for
	American options, some utility functions, and print and summary
	methods for options.
	
	The functions are:

	\tabular{rll}{
	   1-3 	\tab 	\code{GBS*} 					
	 		\tab 	Generalized Block-Scholes option, \cr
	 	 4 	\tab 	\code{BlackScholesOption} 			
	 		\tab 	a synonyme for GBSOption, \cr
	     5 	\tab 	\code{Black76Option} 			
	 		\tab 	options on Futures, \cr
	 	 6 	\tab 	\code{MiltersenSchwartzOption} 	
	 		\tab 	options on commodity futures, \cr
	   7-9	\tab	\code{NDF, CND, CBND}
	  		\tab	distribution functions, \cr
	  	10	\tab	\code{print}
	 		\tab    print method for Options, \cr
	    11	\tab	\code{summary}
	 		\tab	summary method for Options. }
	
}


\usage{
GBSOption(TypeFlag, S, X, Time, r, b, sigma)
GBSGreeks(Selection, TypeFlag, S, X, Time, r, b, sigma)
GBSCharacteristics(TypeFlag, S, X, Time, r, b, sigma)
GBSVolatility(price, TypeFlag, S, X, Time, r, b, tol, maxiter)
BlackScholesOption(\dots)

Black76Option(TypeFlag, FT, X, Time, r, sigma)

MiltersenSchwartzOption(TypeFlag, Pt, FT, X, time, Time, 
	sigmaS, sigmaE, sigmaF, rhoSE, rhoSF, rhoEF, KappaE, KappaF)

NDF(x)
CND(x)
CBND(x1, x2, rho)

\method{print}{option}(x, \dots)
\method{summary}{option}(object, \dots)
}


\arguments{

  	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
  	\item{FT}
  		{[Black76*][MiltersenSchwartz*] -
		the futures price, a numeric value.
		}
  	\item{KappaE, KappaF}{
  		[MiltersenSchwartz*] -
		the speed of mean reversion of the forward interest rate (E),
		the speed of mean reversion of the convenience yield (F), 
		a numeric value.
		}
	\item{maxiter, tol}{
	  	[GBSVolatility*] -
	  	the maximum number of iterations and the tolerance to compute
	  	the root of the GBS volatility equation, see \code{uniroot}.
	  	}
	\item{object}{
		an object of class \code{"option"}.
		}
	\item{price}{
	  	[GBSVolatility*] -
	  	the price of the GBS option, a numerical value.
	  	}
  	\item{Pt}{
  		[MiltersenSchwartz*] -
		the zero coupon bond that expires on the option
		maturity; a numeric value.
		}
  	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
  	\item{rhoSE, rhoSF, rhoEF}{
  		[MiltersenSchwartz*] -
		the correlations 
		between the spot commodity price and the future convenience yield (SE),
		between the spot commodity price and the forward interest rate (SF),
		between the forward interest rate and the future convenience yield (EF), 
		a numeric value.
		}
  	\item{S}{
		the asset price, a numeric value.
		}
  	\item{Selection}{
  		[GBSGreeks] -
    	sensitivity to be computed, one of \code{"delta"}, \code{"gamma"},
    	\code{"vega"}, \code{"theta"}, \code{"rho"}, or \code{"CoC"}, 
    	a string value.
    	}
  	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
  	\item{sigmaS, sigmaE, sigmaF}{
  		[MiltersenSchwartz*] - 
		numeric values, the annualized volatility 
		of the spot commodity price (S),
		of the future convenience yield (E), and
		of the forward interest rate (F),
		e.g. 0.25 means 25\% pa.
		}
	\item{time, Time}{
		the time to maturity measured in years, a numeric value.
		}
  	\item{TypeFlag}{
		a character string either \code{"c"} for a call option or 
		a \code{"p"} for a put option.
		}
	\item{x, x1, x2, rho}{
		[NDF][CND][CBND] -
		the function argument \code{x} for the normal distribution 
		function \code{NDF} and the cumulated normal distribution 
		\code{CND}. The arguments for the bivariate function are
		named \code{x1} and \code{x2}; \code{rho} is the correlation
		coefficient. \cr
		[print] -
		the object \code{x} to be printed.
		}
  	\item{X}{
		the exercise price, a numeric value.
		}
	\item{\dots}{
		arguments to be passed.
		}

}


\value{
  
	\code{GBSOption}\cr
	\code{BlackScholesOption}
	\cr
  	returns the option price, a numeric value. 
  	\cr

  	\code{GBSGreeks}
  	\cr
  	returns the option sensitivity for the selected Greek, a 
  	numeric value.
  	\cr
  
  	\code{GBSCharacteristics}
  	\cr
  	returns a list with the following entries:
  	\code{premium}, the option price,
  	\code{delta}, the delta sensitivity, 
  	\code{gamma}, the gamma sensitivity,
  	\code{theta}, the theta sensitivity,
  	\code{vega}, the vega sensitivity, 
  	\code{rho}, the rho sensitivity,
  	\code{lambda}, the lambda sensitivity.
  	\cr
  	
  	\code{GBSVolatility}
  	\cr
  	returns the GBS option implied volatility for a given price.
  	\cr
  	
  	\code{Black76Option},\cr 
  	\code{MiltersenSchwartzOption}
  	\cr
  	returns the option price, a numeric value.
 
}


\details{

  	\bold{Generalized Black Scholes Options:}
	\cr\cr
  	\code{GBSOption} calculates the option price, \code{GBSGreeks}
  	calculates option sensitivities delta, theta, vega, rho, lambda 
  	and gamma, and \code{GBScharacterisitics} does both.
  	\code{GBSVolatility} computes the implied volatility.
  	\cr
  	Note, that setting \code{b = r} we get Black and Scholes' stock option 
  	model, \code{b = r-q} we get Merton's stock option model with continuous 
  	dividend yield \code{q}, \code{b = 0} we get Black's futures option 
  	model, and \code{b = r-rf} we get Garman and Kohlhagen's currency 
  	option model with foreign interest rate \code{rf}.
  	\cr
  
  		
  	\bold{Options on Futures:}
	\cr\cr
  	The \code{Black76Option} pricing formula is applicable for valuing 
  	European call and European put options on commodity futures. The 
  	exact nature of the underlying commodity varies and may be anything 
  	from a precious metal such as gold or silver to agricultural products.
  	\cr
  	The \code{Miltersen Schwartz Option} model is a three factor model 
  	with stochastic futures prices, term structures and convenience 
  	yields, and interest rates. The model is based on lognormal 
  	distributed commodity prices and normal distributed continuously 
  	compounded forward interest rates and future convenience yields.
  	\cr
  	
	
	\bold{Miltersen Schwartz Options:}
	\cr\cr
	The \code{MiltersenSchwartzOption} function allows for pricing 
	options on commodity futures. The model is a three factor model
	with stochastic futures prices, term structures of convenience 
	yields, and interest rates. The model is based on lognormal
	distributed commodity prices and normal distributed continuously
	compounded forward interest rates and futures convenience yields.
	\cr
  	
  	
  	\bold{Distribution Functions:}
  	\cr\cr
  	The functions \code{NDF}, \code{CND}, and \code{CBND} compute 
  	vlues for the Normal density functions, for the normal 
  	probability function, and for the bivariate normal probability
  	functions. The functions are implemented as described in the
  	book of E.G. Haug.
  	\cr
  	
  	
  	\bold{Print and Summary Method:}
  	\cr\cr
  	Thes are two methods to print and sumarize an object of class
  	\code{"option"}. 
  
}


\note{
  
	The functions implement algorithms to valuate plain vanilla 
  	options and to compute option Greeks as described in Chapter 1 
  	of Haug's Option Guide (1997).
  	
}


\examples{
## All the examples are from Haug's Option Guide (1997)

## CHAPTER 1.1: ANALYTICAL FORMULAS FOR EUROPEAN OPTIONS:

## Black Scholes Option [Haug 1.1.1]
   xmpOptions("\nStart: Generalized Black Scholes Option > ")
   GBSOption(TypeFlag = "c", S = 60, X = 65, Time = 1/4, r = 0.08, 
     b = 0.08, sigma = 0.30)
        
## European Option on a Stock with Cash Dividends [Haug 1.1.2]
   xmpOptions("\nNext: Stock Option with Cash Dividends > ")
   S0  =  100; r  =  0.10; D1  =  D2  =  2; t1  =  1/4; t2  =  1/2
   S  =  S0 - 2*exp(-r*t1) - 2*exp(-r*t2)
   GBSOption(TypeFlag = "c", S = S, X = 90, Time = 3/4, r = r, b = r, 
   	 sigma = 0.25)
   
## Options on Stock Indexes [Haug 1.2.3]
   xmpOptions("\nNext: Options on Stock Indexes > ")
   GBSOption(TypeFlag = "p", S = 100, X = 95, Time = 1/2, r = 0.10, 
     b = 0.10-0.05, sigma = 0.20)  
        
## Option on Futures [Haug 1.1.4]
   xmpOptions("\nNext: Option on Futures > ")
   FuturesPrice  =  19
   GBSOption(TypeFlag = "c", S = FuturesPrice, X = 19, Time = 3/4, 
   	 r = 0.10, b = 0, sigma = 0.28)  
        
## Currency Option [Haug 1.1.5]
   xmpOptions("\nNext: Currency Option > ")
   r  =  0.06; rf  =  0.08
   GBSOption(TypeFlag = "c", S = 1.5600, X = 1.6000, 
     Time = 1/2, r = 0.06, b = 0.06-0.08, sigma = 0.12)
        
## Delta of GBS Option [Haug 1.3.1]
   xmpOptions("\nNext: Delta of GBS Option > ")
   GBSGreeks(Selection = "delta", TypeFlag = "c", S = 105, X = 100, 
     Time = 1/2, r = 0.10, b = 0, sigma = 0.36)

## Gamma of GBS Option [Haug 1.3.3]
   xmpOptions("\nNext: Gamma of GBS Option > ")
   GBSGreeks(Selection = "gamma", TypeFlag = "c", S = 55, X = 60, 
     Time = 0.75, r = 0.10, b = 0.10, sigma = 0.30)
   
## Vega of GBS Option [Haug 1.3.4]
   xmpOptions("\nNext: Vega of GBS Option > ")
   GBSGreeks(Selection = "vega", TypeFlag = "c", S = 55, X = 60, 
     Time = 0.75, r = 0.10, b = 0.10, sigma = 0.30)
   
## Theta of GBS Option [Haug 1.3.5]
    xmpOptions("\nNext: Theta of GBS Option > ")
    GBSGreeks(Selection = "theta", TypeFlag = "p", S = 430, X = 405, 
     Time = 0.0833, r = 0.07, b = 0.07-0.05, sigma = 0.20)
   
## Rho of GBS Option [Haug 1.3.5]
   xmpOptions("\nNext: Rho of GBS Option > ")
   GBSGreeks(Selection = "rho", TypeFlag = "c", S = 72, X = 75, 
     Time = 1, r = 0.09, b = 0.09, sigma = 0.19)
   
## CHAPTER 1.3 OPTIONS SENSITIVITIES:
   
## The Generalized Black Scholes Option Formula
   xmpOptions("\nNext: Generalized Black Scholes Formula > ")
   GBSCharacteristics(TypeFlag = "p", S = 1.5600, X = 1.6000, 
     Time = 1, r = 0.09, b = 0.09, sigma = 0.19)
  
## CHAPTER 1.5: RECENT DEVELOPMENTS IN COMMODITY OPTIONS 

## Miltersen Schwartz Option
## vs. Black76 Option on Futures:
   xmpOptions("\nNext: Miltersen Schwartz Option > ")
   MiltersenSchwartzOption(TypeFlag = "c", Pt = exp(-0.05/4), FT = 95, 
   	 X = 80, time = 1/4, Time = 1/2, sigmaS = 0.2660, sigmaE = 0.2490, 
   	 sigmaF = 0.0096, rhoSE = 0.805, rhoSF = 0.0805, rhoEF = 0.1243, 
   	 KappaE = 1.045, KappaF = 0.200)
   Black76Option(TypeFlag = "c", FT = 95, X = 80, Time = 1/2, r = 0.05, 
   	 sigma = 0.266)  

}


\references{

Black F., Scholes M. (1973);
	\emph{The Pricing of Options and Corporate Liabilities},
	Journal of Political Economy 81, 637--654.

Haug E.G. (1997); 
	\emph{The Complete Guide to Option Pricing Formulas}, 
	Chapter 1, McGraw-Hill, New York.

Hull J.C. (1998);
	\emph{Introduction to Futures and Options Markets},
	Prentice Hall, London.
	
Miltersen K., Schwartz E.S. (1998);
	\emph{Pricing of Options on Commodity Futures with
	Stochastic Term Structuures of Convenience Yields and 
	Interest Rates},
	Journal of Financial and Quantitative Analysis 33, 33--59.

}


\author{

	Diethelm Wuertz for this R-Port.

}


\keyword{math}


\eof
\name{BasicAmericanOptions}

\alias{BasicAmericanOptions}

\alias{RollGeskeWhaleyOption}
\alias{BAWAmericanApproxOption}
\alias{BSAmericanApproxOption}


\title{Valuation of Basic American Options}


\description{

	This is a collection of functions to valuate basic American 
	options. Approximative formulas for American calls are given 
	for the Roll, Geske and Whaley Approximation, for the 
	Barone-Adesi and Whaley Approximation, and for the Bjerksund 
	and Stensland Approximation.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{RollGeskeWhaleyOption} 	
	 		\tab 	Roll, Geske and Whaley Approximation, \cr
	 	2 	\tab 	\code{BAWAmericanApproxOption} 	
	 		\tab 	Barone-Adesi and Whaley Approximation, \cr
	 	3 	\tab 	\code{BSAmericanApproxOption} 	
	 		\tab 	Bjerksund and Stensland Approximation. }
	
}


\usage{
RollGeskeWhaleyOption(S, X, time1, Time2, r, D, sigma) 
BAWAmericanApproxOption(TypeFlag, S, X, Time, r, b, sigma)
BSAmericanApproxOption(TypeFlag, S, X, Time, r, b, sigma) 
}


\arguments{

  	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
  	\item{D}{
    	a single dividend with time to dividend payout \code{t1}.
    	}
  	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
  	\item{S}{
		the asset price, a numeric value.
		}
  	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric value.
		}
	\item{time1, Time2}{
		[RollGeskeWhaley*] - 
		the first value measures time to dividend payout in years, 
		e.g. 0.25 denotes a quarter, and the second value measures 
		time to maturity measured in years, a numeric value; e.g. 
		0.5 means 6 months.
		}
  	\item{TypeFlag}{
		a character string either "c" for a call option or a "p" 
		for a put option.
		}
  	\item{X}{
		the exercise price, a numeric value.
		}

}


\value{
  
  	\code{RollGeskeWhaleyOption} \cr
  	\code{BAWAmericanApproxOption}
  	\cr
  	return the option price, a numeric value.
  	\cr
  	
  	\code{BSAmericanApproxOption}
  	\cr
  	returns a list with the following two elements: \code{Premium} the 
  	option price, and \code{TriggerPrice} the trigger price.  
  	\cr
 
}


\details{
  	
  	\bold{Roll-Geske-Whaley Option:}
  	\cr\cr
  	The function \code{RollGeskeWhaleyOption} valuates American calls 
  	on a stock paying a single dividend with specified time to dividend 
  	payout according to the pricing formula derived by Roll, Geske and 
  	Whaley (1977).
	\cr
	
	
	\code{Approximations for American Options:} 
	\cr\cr
	The function \code{BSAmericanApproxOption} valuates American calls 
	or puts on an underlying asset for a given cost-of-carry rate 
	according to the quadratic approximation method due to Barone-Adesi 
	and Whaley (1987). The function \code{BSAmericanApproxOption} valuates 
	American calls or puts on stocks, futures, and currencies due to 
	the approximation method of Bjerksund and Stensland (1993).
  
}


\note{
  
	The functions implement the algorithms to valuate basic American 
  	options as described in Chapter 1.4 of Haug's Option Guide (1997).
  	
}


\examples{
## All the examples are from Haug's Option Guide (1997)

## CHAPTER 1.4: ANALYTICAL MODELS FOR AMERICAN OPTIONS
       
## Roll-Geske-Whaley American Calls on Dividend Paying 
## Stocks [Haug 1.4.1]
   xmpOptions("\nStart: Roll-Geske-Whaley > ")
   RollGeskeWhaleyOption(S = 80, X = 82, time1 = 1/4, 
     Time2 = 1/3, r = 0.06, D = 4, sigma = 0.30)
   	  
## Barone-Adesi and Whaley Approximation for American 
## Options [Haug 1.4.2] vs. Black76 Option on Futures:
   xmpOptions("\nNext: BAW Approximation > ")
   BAWAmericanApproxOption(TypeFlag = "p", S = 100, 
     X = 100, Time = 0.5, r = 0.10, b = 0, sigma = 0.25)
   Black76Option(TypeFlag = "c", FT = 100, X = 100, 
     Time = 0.5, r = 0.10, sigma = 0.25)  
   	 
## Bjerksund and Stensland Approximation for American 
## Options:
   xmpOptions("\nNext: BS Approximation > ")
   BSAmericanApproxOption(TypeFlag = "c", S = 42, X = 40, 
	 Time = 0.75, r = 0.04, b = 0.04-0.08, sigma = 0.35)
}


\references{
Barone-Adesi G., Whaley R.E. (1987);
	\emph{Efficient Analytic Approximation of American Option Values},
	Journal of Finance 42, 301--320.
	
Bjerksund P., Stensland G. (1993);
	\emph{Closed Form Approximation of American Options},
	Scandinavian Journal of Management 9, 87--99.
	
Geske R. (1979);
	\emph{A Note on an Analytical Formula for Unprotected
	American Call Options on Stocks with known Dividends},
	Journal of Financial Economics 7, 63--81.

Haug E.G. (1997); 
	\emph{The Complete Guide to Option Pricing Formulas}, 
	Chapter 1, McGraw-Hill, New York.

Roll R. (1977);
	\emph{An Analytic Valuation Formula for Unprotected
	American Call Options on Stocks with known Dividends},
	Journal of Financial Economics 5, 251--258.
}


\author{

	Diethelm Wuertz for this R-Port.

}


\keyword{math}


\eof
\name{BinomialTreeOptions}

\alias{BinomialTreeOptions}

\alias{CRRBinomialTreeOption}
\alias{JRBinomialTreeOption}
\alias{TIANBinomialTreeOption}
\alias{BinomialTreeOption}
\alias{BinomialTreePlot}


\title{Binomial Tree Option Model}


\description{
  
	This is a collection of functions to valuate options in the 
	framework of the Binomial tree option approach.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{CRRBinomialTreeOption} 	
	 		\tab 	CRR Binomial Tree Option, \cr
	 	2 	\tab 	\code{JRBinomialTreeOption} 	
	 		\tab 	JR Binomial Tree Option, \cr
	 	3 	\tab 	\code{TIANBinomialTreeOption} 	
	 		\tab 	TIAN Binomial Tree Option, \cr
	 	4 	\tab 	\code{BinomialTreeOption} 		
	 		\tab 	Binomial Tree Option, \cr
	 	5 	\tab 	\code{BinomialTreePlot} 		
	 		\tab 	Binomial Tree Plot. }
  	
}


\usage{
CRRBinomialTreeOption(TypeFlag, S, X, Time, r, b, sigma, n)
JRBinomialTreeOption(TypeFlag, S, X, Time, r, b, sigma, n)
TIANBinomialTreeOption(TypeFlag, S, X, Time, r, b, sigma, n)

BinomialTreeOption(TypeFlag, S, X, Time, r, b, sigma, n)
BinomialTreePlot(BinomialTreeValues, dx = -0.025, dy = 0.4, 
	cex = 1, digits = 2, \dots)
}


\arguments{
  
	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{BinomialTreeValues}{
	    the return value from the \code{BinomialTreeOption} function.
	    }
	\item{cex}{
	    a numerical value giving the amount by which the plotting text
        and symbols should be scaled relative to the default.
		}
	\item{digits}{
	    an integer value, how many digits should be displayed in the
	    option tree?
	    }
	\item{dx, dy}{
	    numerical values, an offset fine tuning for the placement of
	    the option values in the option tree.
	    }
	\item{n}{
		number of time steps; an integer value.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
	\item{S}{
		the asset price, a numeric value.
		}
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric 
		value; e.g. 0.5 means 6 months.
		}
	\item{TypeFlag}{
		a character string either \code{"ce"}, \code{"ca"} for an 
		European or American call option or a \code{"pe"}, \code{"pa"} 
		for a put option, respectively.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
	\item{\dots}{
	    arguments to be passed.
	    }

}


\details{
  
	\bold{CRR Binomial Tree Model:}
	\cr\cr
	Binomial models were first suggested by Cox, Ross and Rubinstein (1979),
	CRR, and then became widely used because of its intuition and easy 
	implementation. Binomial trees are constructed on a discrete-time 
	lattice. With the time between two trading events shrinking to zero, 
	the evolution of the price converges weakly to a lognormal diffusion. 
	Within this mode the European options value converges to the value 
	given by the Black-Scholes formula. 
	\cr
	
	
	\bold{JR Binomial Tree Model:}
	\cr\cr
	There exist many extensions of the CRR model. Jarrow and Rudd (1983), 
	JR, adjusted the CRR model to account for the local drift term. They 
	constructed a binomial model where the first two moments of the 
	discrete and continuous time return processes match. As a consequence 
	a probability measure equal to one half results. Therefore the CRR and
	JR models are sometimes atrributed as equal jumps binomial trees and
	equal probabilities binomial trees.
	\cr
	

	\bold{TIAN Binomial Tree Model:}
	\cr\crTian (1993) suggested to match discrete and continuous local 
	moments up to third order. 
	\cr
	Leisen and Reimer (1996) proved that the order of convergence in 
	pricing European options for all three methods is equal to one, and
	thus the three models are equivalent.

}


\note{

	Note, the \code{BinomialTree} and \code{BinomialTreePlot} are preliminary 
	implementations.
		
}


\value{

	The option price, a numeric value.
	
}


\examples{
## Cox-Ross-Rubinstein Binomial Tree Option Model:
 
   # Example 14.1 from Hull's Book:
   xmpOptions("\nStart: CRR Binomial Tree > ")
   CRRBinomialTreeOption(TypeFlag = "pa", S = 50, X = 50, 
     Time = 5/12, r = 0.1, b = 0.1, sigma = 0.4, n = 5)
   # Example 3.1.1 from Haug's Book:
  	CRRBinomialTreeOption(TypeFlag = "pa", S = 100, X = 95, 
		Time = 0.5, r = 0.08, b = 0.08, sigma = 0.3, n = 5)
   # A European Call - Compare with Black Scholes: 
   CRRBinomialTreeOption(TypeFlag = "ce", S = 100, X = 100, 
   	 Time = 1, r = 0.1, b = 0.1, sigma = 0.25, n = 50)
   GBSOption(TypeFlag = "c", S = 100, X = 100, 
   	 Time = 1, r = 0.1, b = 0.1, sigma = 0.25)$price
   	 
## CRR - JR - TIAN Model Comparison
   	 
   # Hull's Example as Function of "n":
   xmpOptions("\nNext: Convergence Behavior > ")
   par(mfrow = c(2, 1), cex = 0.7)
   steps = 50
   CRROptionValue =  JROptionValue = TIANOptionValue = 
     rep(NA, times = steps)
   for (n in 3:steps) { 
   	 CRROptionValue[n] = CRRBinomialTreeOption(TypeFlag = "pa", S = 50, 
	   X = 50, Time = 0.4167, r = 0.1, b = 0.1, sigma = 0.4, n = n)
	 JROptionValue[n] = JRBinomialTreeOption(TypeFlag = "pa", S = 50, 
	   X = 50, Time = 0.4167, r = 0.1, b = 0.1, sigma = 0.4, n = n)	
	 TIANOptionValue[n] = TIANBinomialTreeOption(TypeFlag = "pa", S = 50, 
	   X = 50, Time = 0.4167, r = 0.1, b = 0.1, sigma = 0.4, n = n) }			
   plot(CRROptionValue[3:steps], type = "l", col = "red", 
   	 ylab = "Option Value")
   lines(JROptionValue[3:steps], col = "green")
   lines(TIANOptionValue[3:steps], col = "blue")
   # Add Result from BAW Approximation:
   BAWValue =  BAWAmericanApproxOption(TypeFlag = "p", S = 50, X = 50, 
   	 Time = 0.4167, r = 0.1, b = 0.1, sigma = 0.4)
   abline(h = BAWValue, lty = 3)
   title(main = "Convergence")
   data.frame(CRROptionValue, JROptionValue, TIANOptionValue)
   
## Plot CRR Option Tree:

   # Again Hull's Example:
   xmpOptions("\nNext: Plot CRR Option Tree > ")
   CRRTree = BinomialTreeOption(TypeFlag = "pa", S = 50, X = 50, 
	 Time = 0.4167, r = 0.1, b = 0.1, sigma = 0.4, n = 5)
   BinomialTreePlot(CRRTree, dy = 1, cex = 0.8, ylim = c(-6, 7),
   	 xlab = "n", ylab = "Option Value")
   title(main = "Option Tree")

}


\references{
	
Broadie M., Detemple J. (1994);
	\emph{American Option Evaluation: New Bounds, Approximations, 
		and a Comparison of Existing Methods}, 
	Working Paper, Columbia University, New York. 

Cox J., Ross S.A., Rubinstein M. (1979);
	\emph{Option Pricing: A Simplified Approach}, 
	Journal of Financial Economics 7, 229--263. 

Haug E.G. (1997); 
	\emph{The complete Guide to Option Pricing Formulas}, 
	McGraw-Hill, New York.
	
Hull J.C. (1998);
	\emph{Introduction to Futures and Options Markets},
	Prentice Hall, London.
	
Jarrow R., Rudd A. (1983); 
	\emph{Option Pricing}, 
	Homewood, Illinois, 183--188. 
	
Leisen D.P., Reimer M., (1996); 
	\emph{Binomial Models for Option Valuation -- Examining and 
		Improving Convergence}, 
	Applied Mathematical Finanace 3, 319--346.

Tian Y. (1993);
	\emph{A Modified Lattice Approach to Option Pricing}, 
	Journal of Futures Markets 13, 563--577. 

}


\author{

 	Diethelm Wuertz for this R-Port.
 	
}


\keyword{math}


\eof
\name{MultExercisesOptions}

\alias{MultExercisesOptions}

\alias{ExecutiveStockOption}
\alias{ForwardStartOption}
\alias{RatchetOption}
\alias{TimeSwitchOption}
\alias{SimpleChooserOption}
\alias{ComplexChooserOption}
\alias{OptionOnOption}
\alias{WriterExtendibleOption}
\alias{HolderExtendibleOption}


\title{Valuation of Mutiple Exercises Options}


\description{
  
	This is a collection of functions to valuate multiple exercise 
	options. Multiple exercises options, as the name implies, are 
	options whose payoff is based on multiple exercise dates.
	\cr
	
	The functions are:

	\tabular{rll}{
	  	1	\tab 	\code{ExecutiveStockOption} 		
	  		\tab 	Executive Stock Option, \cr
	  	2 	\tab 	\code{ForwardStartOption}		
	  		\tab 	Forward Start Option, \cr
	  	3 	\tab 	\code{RatchetOption} 			
	  		\tab 	Ratchet Option, \cr
	  	4 	\tab 	\code{TimeSwitchOption} 		
	  		\tab 	Time Switch Option, \cr
	  	5 	\tab 	\code{SimpleChooserOption} 	
	  		\tab 	Simple Chooser Option, \cr
	  	6 	\tab 	\code{ComplexChooserOption} 	
	  		\tab 	Complex Chooser Option, \cr
	  	7 	\tab 	\code{OptionOnOption} 			
	  		\tab 	Option On Option, \cr
	  	8 	\tab 	\code{WriterExtendibleOption} 	
	  		\tab 	Writer Extendible Option, \cr
	  	9 	\tab 	\code{HolderExtendibleOption} 	
	  		\tab 	Holder Extendible Option. }
	
}


\usage{
ExecutiveStockOption(TypeFlag, S, X, Time, r, b, sigma, lambda)
ForwardStartOption(TypeFlag, S, alpha, time1, Time2, r, b, sigma) 
RatchetOption(TypeFlag, S, alpha, time1, Time2, r, b, sigma)
TimeSwitchOption(TypeFlag, S, X, Time, r, b, sigma, A, m, dt)
SimpleChooserOption(S, X, time1, Time2, r, b, sigma) 
ComplexChooserOption(S, Xc, Xp, Time, Timec, Timep, r, b, sigma, doprint = FALSE)
OptionOnOption(TypeFlag, S, X1, X2, time1, Time2, r, b, sigma, doprint = FALSE)
WriterExtendibleOption(TypeFlag, S, X1, X2, time1, Time2, r, b, sigma)
HolderExtendibleOption(TypeFlag, S, X1, X2, time1, Time2, r, b, sigma, A)
}


\arguments{    
	
	\item{A}{
		[HolderExtendible*] - 
		defined by the amount \code{A*dt} the investor receives
		at maturity time \code{Time} for each time interval
		\code{deltat} the corresponding asset price has exceeded 
		the exercise price \code{X}, in the case of a call
		option, or the corresponding asset price has been below 
		the exercise price \code{X}, in the case of a put
		option. A numeric value.
		}
  	\item{alpha}{
  		[Ratchet*] - 
		the exercise price is \code{alpha} times the asset price
		\code{S} after the known time \code{time}. \code{alpha}
		is a numeric value. If \code{alpha} is less than unity,
		the call (put) will start \code{100*(1-alpha)} percent in 
		the money (out-of-the-money); if  \code{alpha} is unity, 
		the option will start at the money; and if \code{alpha} 
		is larger than unity, the call (put) will start 
		\code{100*(alpha-1)} percentage out of the money 
		(in-the-money).
		}
  	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{doprint}{
		a logical. Should the critical value \code{I} be printed?
		By defaut \code{FALSE}.
		}	
  	\item{dt}{
		the time interval; a numeric value.
		}
	\item{lambda}{
	  	the jump rate pa.
	  	}
  	\item{m}{
		defined by the number of time units where the option
		has already fulfilled the thresold condition. This
		applies to cases, for which some of the option's total
		lifetime has already passed. An integer value.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}	
	\item{S}{
		the asset price, a numeric value.
		}
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}	
	\item{Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{Timec, Timep}{
		[ComplexChooser*] -
		decision time measured in years, e.g. 0.5 means 6 months.  
		\code{Timec}, is the time to maturity of the call option, 
		\code{Timep}, is the time to maturity of the put option, 
		both also measured in years. Numeric values.
		}
	\item{time1, Time2}{
		the time to maturity, \code{Time1}, measured in years, 
		e.g. 0.5 means 6 months, and the elapsed time in the 
		future, \code{Time2}. In detail, the forward start option 
		with time to maturity \code{Time1} starts at-the-money or 
		proportinally in-the-money or out-of-the-money after this
		elapsed time \code{Time2} in the future.
		}
	\item{TypeFlag}{
		usually a character string either \code{"c"} for a call option 
		or a \code{"p"} for a put option;\cr
		[OptionOnOption] -  
		a character string either 
		\code{"cc"} for a call-on-call option, or
		\code{"cp"} for a call-on-put option, or
		\code{"pc"} for a put-on-call option, or 
		\code{"pp"} for a put-on-put option.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
	\item{Xc, Xp}{
		[ComplexChooser*] -
		the exercise price of the call option, \code{Xc}, and the 
		exercise price of the put option, \code{Xp}, numeric 
		values.
		}
	\item{X1, X2}{
		the exercise price of the underlying option, \code{X1}, and
		the exercise price of the option on the option, \code{X2},
		numeric values.
		}
	
}


\details{

  	\bold{Executive Stock Options:}
	\cr\cr
  	Executive stock options are usually at-the-money options that are 
  	issued to motivate employees to act in the best interest of the 
  	company. They cannot be sold and often last as long as 10 or 15 
  	years. The executive model takes into account that an employee often 
  	looses their options when they leave the company before expiration. 
  	The value of an executive option equals the standard Black-Scholes 
  	model multiplied by the probability that the employee will stay with 
  	the firm until the option expires. Executive stock options can be priced
  	analytically using a model published by Jennergren and Naslund (1993). 
  	\cr
  	[Haug's Book, Chapter 2.1]
  	\cr
  	

  	\bold{Forward Start Options:}
	\cr\cr
  	A forward start option is an option which is paid for today, but 
  	will start at some determined time in the future known as the issue 
  	date. The option usually starts at-the-money or proportionally in 
  	or out-of-the-money at a future date. The strike is set to a positive 
  	constant a times the asset price S at a future date. If a is less 
  	than one, the call (put) will start 1 - a percent in-the-money 
  	(out-of-the-money); if a is one, the option will start at-the-money; 
  	and if a is larger than one, the call (put) will start a - 1 percent 
  	out-of-the-money (in-the-money).[1] Forward start options can be 
  	priced analytically using a model published by Rubinstein (1990).
  	\cr
	[Haug's Book, Chapter 2.2]
	\cr

	
	\bold{Ratchet [Compound] Options:}
	\cr\cr
	A compound option is an option on an option. Therefore, when one 
	option is exercised, the underlying security is another option. 
	There are four types of possible compound options: a call on a call, 
	a call on a put, a put on a call, and a put on a put. The owner of 
	a compound option has until the expiration date of the compound 
	option to determine whether to exercise the compound option. If 
	exercised, the owner will receive the underlying option with its 
	own exercise price and time until expiration. If the underlying 
	option is exercised, the owner will receive the underlying security. 
	European compound options can be priced analytically using a model 
	published by Rubinstein (1991). A binomial lattice is used for the 
	numerical calculation of an American or European style exchange option.
	A ratchet option is also called sometimes a "moving strike option" 
	or "cliquet option".
	\cr
	[Haug's Book, Chapter 2.3]
	\cr
	

  	\bold{Time-Switch Options:}
	\cr\cr
  	For a discrete time-switch call (put) option, the holder receives an 
  	amount ADt at expiration for each time interval, Dt, the corresponding 
  	asset price has been above (below) the strike price. If some of the 
  	options total lifetime has passed, it is required to add a fixed 
  	amount to the pricing formula. Discrete time-switch options can be 
  	priced analytically using a model published by Pechtl (1995).
  	\cr
  	[Haug's Book, Chapter 2.4]
  	\cr
  	

  	\bold{Simple Chooser Options:}
	\cr\cr
  	A chooser option allows the holder to determine at some date, after the 
  	trade date, whether the option becomes a plain vanilla call or put. 
  	Chooser options are also called "as you like it" options. Chooser 
  	options are useful for hedging a future event that might not occur. 
  	Due to their increased flexibility, chooser options are more expensive 
  	than plain vanilla options. It is assumed at the options expiration 
  	date that a holder of the chooser option will choose the more valuable 
  	of the put or call option. The less valuable option that was not chosen 
  	will become worthless. Chooser options can be priced analytically using 
  	a model introduced by Rubinstein (1991).
  	\cr
  	[Haug's Book, Chapter 2.5.1]
  	\cr

  	
  	\bold{Complex Chooser Options:}
	\cr\cr
  	A complex chooser option allows the holder to determine at some date, 
  	after the trade date, whether the option is to be a standard call 
  	chooser model, a complex chooser option will be more expensive than 
  	a plain vanilla option. Complex chooser options can be priced analytically 
  	using a model introduced by Rubinstein (1991).
  	\cr
  	[Haug's Book, Chapter 2.5.2]
  	\cr
  
  	
  	\bold{Option On Options:}
  	\cr\cr
  	This derivative prices options on options. An option on an option is more 
  	expensive to purchase than the underlying option itself, as the purchaser 
  	has received a price guarantee and effectively extended the life of the 
  	option. These options provide the benefit of a guaranteed price for the 
  	option at a date in the future. Options on Options can be prices as
  	published by Geske (1977). His model was later extended and discussed 
  	by Geske (1979), Hodges and Selby (1987), and Rubinstein (1991). 
  	\cr
  	[Haug's Book, Chapter 2.6]
  	\cr

  	
  	\bold{Writer [Holder] Extendible Options:}
	\cr\cr
  	Writer extendible options can be found embedded in various financial 
  	contracts. For example, corporate warrants often give the issuing 
  	firm the right to extend the life of the warrants. These options can 
  	be exercised at their initial maturity, but are extended to a new 
  	maturity if they are out-of-the-money at initial maturity. Discrete 
  	time-switch options can be priced analytically using a model published 
  	by Longstaff (1995).
  	\cr
  	[Haug's Book, Chapter 2.6]

}


\note{
	
	Options on options are also known as compound options or as 
	mother-and-daughter options. 
	
}


\value{

	The option price, a numeric value.
	
}


\references{
Geske R. (1977);
	\emph{The Valuation of Corporate Liabilities as Compound Options},
	Journal of Financial and Quantitative Analysis, 541--552.

Geske R. (1979);
	\emph{The Valuation of Compound Options},
	Journal of Financial Economics 7, 63--81.
	
Haug E.G. (1997);
	\emph{The complete Guide to Option Pricing Formulas}, 
	Chapter 2.8.1, McGraw-Hill, New York.
	
Hodges S.D., Selby J.P. (1987);
	\emph{On the Evaluation of Compound Options};
	Management Science 33, 347--355.

Jennergren L.P., Naslund B. (1993); 
	\emph{A Comment on Valuation of Executive Stock Options and the 
	FASB Proposal}, 
	The Accounting Review 68, 179, 1993.

Longstaff F.A. (1990);
	\emph{Pricing Options with Extendible Maturities: Analysis
	and Applications},
	Journal of Finance 45, 474--491.
	
Pechtl A. (1990); 
	\emph{Classified Information},
	Risk Magazine 8.
	
Rubinstein, M. (1990); 
	\emph{Pay Now, Choose Later}, 
	Risk Magazine 3.
	
Rubinstein M. (1991); 
	\emph{Options for the Undecide}, 
	Risk Magazine 4.
	
Rubinstein M. (1991);
	\emph{Double Trouble};
	Risk Magazine 5.
	
}


\author{

	Diethelm Wuertz for this R-Port.
	
}


\examples{
## Examples from Chapter 2.1 - 2.7 in E.G. Haug's Option Guide (1997)

## ExecutiveStockOption [2.1]:
   xmpOptions("\nStart: Executive Stock Option > ")
   ExecutiveStockOption(TypeFlag = "c", S = 60, X = 64, Time = 2, 
	r = 0.07, b = 0.07-0.03, sigma = 0.38, lambda = 0.15) 
	
## ForwardStartOption [2.2]:
   xmpOptions("\nNext: Forward Start Option > ")
   ForwardStartOption(TypeFlag = "c", S = 60, alpha = 1.1, 
     time1 = 1, Time2 = 1/4, r = 0.08, b = 0.08-0.04, sigma = 0.30)
     
## Ratchet Option [2.3]:
   xmpOptions("\nNext: Ratchet Option > ")
   RatchetOption(TypeFlag = "c", S = 60, alpha = 1.1, time1 = c(1.00, 0.75), 
	 Time2 = c(0.75, 0.50), r = 0.08, b = 0.04, sigma = 0.30)
	 
## Time Switch Option [2.4]:
   xmpOptions("\nNext: Time Switch Option > ")
   TimeSwitchOption(TypeFlag = "c", S = 100, X = 110, Time = 1, 
	r = 0.06, b = 0.06, sigma = 0.26, A = 5, m = 0, dt = 1/365)
	
## Simple Chooser Option [2.5.1]:
   xmpOptions("\nNext: Simple Chooser Option > ")
   SimpleChooserOption(S = 50, X = 50, time1 = 1/4, Time2 = 1/2, 
     r = 0.08, b = 0.08, sigma = 0.25)  
        
## Complex Chooser Option [2.5.2]:
   xmpOptions("\nNext: Complex Chooser Option > ")
   ComplexChooserOption(S = 50, Xc = 55, Xp = 48, Time = 0.25, 
     Timec = 0.50, Timep = 0.5833, r = 0.10, b = 0.1-0.05, 
     sigma = 0.35, doprint = TRUE)
	 
## Option On Option [2.6]:
   xmpOptions("\nNext: Option On Option > ")
   OptionOnOption(TypeFlag = "pc", S = 500, X1 = 520, X2 = 50, 
     time1 = 1/2, Time2 = 1/4, r = 0.08, b = 0.08-0.03, sigma = 0.35)
	
## Holder Extendible Option [2.7.1]:
   xmpOptions("\nNext: Holder Extendible Option > ")
   HolderExtendibleOption(TypeFlag = "c", S = 100, X1 = 100, 
     X2 = 105, time1 = 0.50, Time2 = 0.75, r = 0.08, b = 0.08, 
     sigma = 0.25, A = 1)
	 
## Writer Extendible Option [2.7.2]:
   xmpOptions("\nNext: Writer Extendible Option > ")
   WriterExtendibleOption(TypeFlag = "c", S = 80, X1 = 90, X2 = 82,
	 time1 = 0.50, Time2 = 0.75, r = 0.10, b = 0.10, sigma = 0.30)
 
}


\keyword{math}


\eof
\name{MultAssetsOptions}

\alias{MultAssetsOptions}

\alias{TwoAssetCorrelationOption}
\alias{EuropeanExchangeOption}
\alias{AmericanExchangeOption}
\alias{ExchangeOnExchangeOption}
\alias{TwoRiskyAssetsOption}
\alias{SpreadApproxOption}


\title{Valuation of Mutiple Assets Options}


\description{

	This is a collection of functions to valuate multiple asset options.
	Multiple asset options, as the name implies, are options whose payoff 
  	is based on two (or more) assets. The two assets are associated with 
  	one another through their correlation coefficient.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{TwoAssetCorrelationOption} 	
	 		\tab  	Two Asset Correlation Option, \cr
	 	2 	\tab 	\code{EuropeanExchangeOption} 		
	 		\tab  	Exchange-One-Asset-For-Another ..., \cr
	 	3 	\tab 	\code{AmericanExchangeOption} 		
	 		\tab  	... European or American Option, \cr
	 	4 	\tab 	\code{ExchangeOnExchangeOption} 	
	 		\tab  	Exchange Option on an Exchange Option, \cr
	 	5 	\tab 	\code{TwoRiskyAssetsOption} 		
	 		\tab  	Option on the Min/Max of 2 Risky Assets, \cr
	 	6 	\tab 	\code{SpreadApproxOption} 			
	 		\tab  	Spread Option Approximation. }
	
}


\usage{
TwoAssetCorrelationOption(TypeFlag, S1, S2, X1, X2, Time, 
 	r, b1, b2, sigma1, sigma2, rho) 
EuropeanExchangeOption(S1, S2, Q1, Q2, Time, r, b1, b2, 
	sigma1, sigma2, rho)
AmericanExchangeOption(S1, S2, Q1, Q2, Time, r, b1, b2, 
	sigma1, sigma2, rho, doprint)
ExchangeOnExchangeOption(TypeFlag, S1, S2, Q, time1, Time2, r, 
	b1, b2, sigma1, sigma2, rho)
TwoRiskyAssetsOption(TypeFlag, S1, S2, X, Time, r, b1, 
	b2, sigma1, sigma2, rho)
SpreadApproxOption(TypeFlag, S1, S2, X, Time, r, sigma1, 
	sigma2, rho)
}


\arguments{    
	
	\item{b1, b2}{
		the annualized cost-of-carry rate for the first and second 
		asset, a numeric value; e.g. 0.1 means 10\% pa.
		}
	\item{doprint}{
		[AmericanExchange*] - 
		a logical flag. Should the trigger price be printed?
		By default \code{FALSE}.
		}
	\item{Q, Q1, Q2}{
		additionally , quantity of the first and second asset.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% p.a.
		}	
	\item{rho}{
		the correlation coefficient between the returns on the two 
		assets.
		}
	\item{S1, S2}{
		the first and second asset price, numeric values.
		}	
	\item{sigma1, sigma2}{
		the annualized volatility of the first and second underlying 
		security, a numeric value; e.g. 0.3 means 30\% volatility p.a.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{time1, Time2}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{TypeFlag}{
		usually a character string either \code{"c"} for a call option 
		or a \code{"p"} for a put option, except for \cr
		[ExchangeOnExchange*] -
		a character string either, \cr
		\code{"1"} denotes: option to exchange \code{Q*S2} for the option
		to exchange \code{S2} for \code{S1}, \cr
		\code{"2"} denotes option to exchange the option to exchange \code{S2} 
		for \code{S1}, in return for \code{Q*S2}, \cr
		\code{"3"} denotes: option to exchange \code{Q*S2} for the option
		to exchange \code{S1} for \code{S2}, \cr
		\code{"4"} denotes option to exchange the option to exchange \code{S1} 
		for \code{S2}, in return for \code{Q*S2}; \cr
		[TwoRiskyAssets*] -
		a character string either, \cr
		\code{"cmin"} denotes: call on the minimum, \cr
		\code{"cmax"} denotes: call on the maximum, \cr
		\code{"pmin"} denotes: call on the minimum, \cr
		\code{"pmax"} denotes: call on the maximum of two risky assets.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
	\item{X1, X2}{
		the first and second exercise price, numeric values.
		}	
			
}


\details{

  	\bold{Two-Asset Correlation Options:}
	\cr\cr
  	A two asset correlation options have two underlying assets and two strike 
  	prices. A two asset correlation call option on two assets S1 and S2 with 
  	a strike prices X1 and X2 has a payoff of max(S2-X2,0) if S1>X1 and 0 
  	otherwise, and a put option has a payoff of max(X2-S2,0) if S1<X1 and 0 
  	otherwise. Two asset correlation options can be priced analytically using 
  	a model introduced by Zhang (1995).
  	\cr
  	[Haug's Book, Chapter 2.8.1]
  	\cr
  	

  	\bold{Exchange-One-Asset-For-Another Options:}
	\cr\cr
  	The exchange option gives the holder the right to exchange one asset for 
  	another. The payoff for this option is the difference between the prices 
  	of the two assets at expiration. The analytical calculation of European 
  	exchange option is based on a modified Black Scholes formula originally 
  	introduced by Margrabe (1978). A binomial lattice is used for the numerical 
  	calculation of an American or European style exchange option.
  	\cr
  	[Haug's Book, Chapter 2.8.2]
  	\cr
  	

  	\bold{Exchange-On-Exchange Options:}
	\cr\cr
  	Exchange options on exchange options can be found embedded in many 
  	sequential exchange opportunities [1]. As an example, a bond holder 
  	converting into a stock, later exchanging the shares received for stocks 
  	of an acquiring firm. This complex option can be priced analytically 
  	using a model introduced by Carr (1988).
  	\cr
  	[Haug's Book, Chapter 2.8.3]
  	\cr
  	

  	\bold{Portfolio Options:}
	\cr\cr
  	A portfolio option is an American (or European) style option on the 
  	maximum of the sum of the prices of two assets and a fixed strike price. 
  	A portfolio call option on two assets S1 and S2 with a strike price X 
  	has a payoff of max((S1+S2)-X,0) and a put option has a payoff of 
  	max((X-(S1+S2),0). A binomial lattice is used for the numerical 
  	calculation of an American or European style portfolio options.
  	\cr
  	

  	\bold{Rainbow Options:}
	\cr\cr
  	A rainbow option is an American (or European) style option on the maximum 
  	(or minimum) of two underlying assets. These types of rainbow options are 
  	generally referred to as two-color rainbow options. There are four general 
  	types of two-color rainbow options: maximum or best of two risky assets, 
  	the minimum or worst of two risky assets, the better of two risky assets, 
  	and the worse of two risky assets. A maximum rainbow call option on two 
  	assets S1 and S2 with a strike price X has a payoff of max(max(S1,S2)-X,0) 
  	and a put option has a payoff of max(X-max(S1,S2),0). A minimum rainbow 
  	call option on two assets S1 and S2 with a strike X has a payoff of 
  	max(min(S1,S2)-X,0) and a put option has a payoff of max(X-min(S1,S2),0). 
  	Set the Strike parameter to a very small number (1e-8) to calculate better 
  	and worse rainbow option types. The analytical calculation of European 
  	rainbow option is based on Rubinsteins (1991) model. A binomial 
  	lattice is used for the numerical calculation of an American or European 
  	style rainbow options.
  	\cr
  	

  	\bold{Spread Options:}
	\cr\cr
  	A spread option is a standard option on the difference of the values of 
  	two assets. Spread options a related to exchange options. If the strike 
  	price is set to zero, a spread option is equivalent to an exchange option. 
  	A spread call option on two assets S1 and S2 with a strike price X has 
  	a payoff of max(S1-S2-X,0) and a put option has a payoff of max(X-S1+S2,0). 
  	The analytical calculation of European spread option is based on 
  	Gauss-Legendre integration and the Black-Scholes model. A binomial 
  	lattice is used for the numerical calculation of an American or European 
  	style spread options.
  	\cr
  	[Haug's Book, Chapter 2.8.5]
  	\cr
  	

  	\bold{Dual Strike Options:}
	\cr\cr
  	A dual strike option is an American (European) option whose payoff 
  	involves receiving the best payoff of two standard American (European) 
  	style plain options. These options have two underlying assets and two 
  	strike prices. The payoff of a dual strike call option is the maximum 
  	of asset one minus strike one or asset two minus strike two. The payoff 
  	of a dual strike put option is the maximum of strike one minus asset one 
  	or strike two minus asset two. The payoff of a reverse dual strike call 
  	option is the maximum of asset one minus strike one or strike two minus 
  	asset two. The payoff of a reverse dual strike put option is the maximum 
  	of strike one minus asset one or asset two minus strike two. A binomial 
  	lattice is used for the numerical calculation of an American or European 
  	style dual strike and reverse dual strike options.

}


\note{
  
	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 2.8 of Haug's Book (1997).
  	
}


\value{

	The option price, a numeric value.

}


\references{
Black F. (1976);
	\emph{The Pricing of Commodity Contracs},
	Journal of Financial Economics 3, 167--179.

Boyle P.P., Evnine J., Gibbs S. (1989);
	\emph{Numerical Evaluation of Multivariate Contingent Claims},
	Review of Financial Studies 2, 241--250.

Boyle P.P., Tse Y.K. (1990);
	\emph{An Algorithm for Computing Values of Options on the
		Maximum or Minimum of Several Assets},
	Journal of Financial and Quantitative Analysis 25, 215--227.

Carr P.P. (1988)
	\emph{The Valuation of Sequential Exchange Opportunities},
	Journal of Finance 43, 1235--1256.
	
Haug E.G. (1997); 
	\emph{The Complete Guide to Option Pricing Formulas}, 
	McGraw-Hill, New York.

Johnson H. (1987)
	\emph{Options on the Maximum or the Minimum of Several Assets},
	Journal of Financial and Quantitative Analysis 22, 277--283.
		
Kirk E. (1995);
	\emph{Correlation in the Energy Markets},
	in: Managing Energy Price Risk,
	Risk Publications and Enron, London, pp. 71--78.
	
Margrabe W. (1998);
	\emph{The Value of an Option to Exchange one Asset for Another},
	Journal of Finance 33, 177--186.
	
Rich D.R, Chance D.M. (1993);
	\emph{An Alternative Approach to the Pricing of Options on 
		Multiple Assets},
	Journal of Financial Engineering 2, 271--285.

Rubinstein M. (1991)
	\emph{Somewhere over the Rainbow},
	Risk Magazine 4, 10.
	
Stulz R.M. (1982);
	\emph{Options on the Minimum or Maximum of Two Risky Assets},
	Journal of Financial Economics 10, 161--185.
	
Zhang P.G. (1995); 
	\emph{Correlation Digital Options}
	Journal of Financial Engineering 3, 5. 
}



\author{

	Diethelm Wuertz for this R-Port.

}


\examples{
## Examples from Chapter 2.8 in E.G. Haug's Option Guide (1997)

## Two Asset Correlation Options [2.8.1]:
   xmpOptions("\nStart: Two Asset Correlation Option > ")
   TwoAssetCorrelationOption(TypeFlag = "c", S1 = 52, S2 = 65, 
     X1 = 50, X2 = 70, Time = 0.5, r = 0.10, b1 = 0.10, b2 = 0.10, 
     sigma1 = 0.2, sigma2 = 0.3, rho = 0.75) 

## European Exchange Options [2.8.2]: 
   xmpOptions("\nNext: European Exchange Option > ")
   EuropeanExchangeOption(S1 = 22, S2 = 0.20, Q1 = 1, Q2 = 1, 
     Time = 0.1, r = 0.1, b1 = 0.04, b2 = 0.06, sigma1 = 0.2, 
     sigma2 = 0.25, rho = -0.5)
	 
## American Exchange Options [2.8.2]:
   xmpOptions("\nNext: American Exchange Option > ")
   AmericanExchangeOption(S1 = 22, S2 = 0.20, Q1 = 1, Q2 = 1, 
     Time = 0.1, r = 0.1, b1 = 0.04, b2 = 0.06, sigma1 = 0.2, 
     sigma2 = 0.25, rho = -0.5)

## Exchange Options On Exchange Options [2.8.3]:
   xmpOptions("\nNext: Exchange On Exchange Option > ")
   for (flag in 1:4) print(
   ExchangeOnExchangeOption(TypeFlag = as.character(flag), 
     S1 = 105, S2 = 100, Q = 0.1, time1 = 0.75, Time2 = 1.0, r = 0.1, 
     b1 = 0.10, b2 = 0.10, sigma1 = 0.20, sigma2 = 0.25, rho = -0.5))

## Two Risky Assets Options [2.8.4]:
   xmpOptions("\nNext: Two Risky Assets Option > ")
   TwoRiskyAssetsOption(TypeFlag = "cmax", S1 = 100, S2 = 105, 
     X = 98, Time = 0.5, r = 0.05, b1 = -0.01, b2 = -0.04, 
     sigma1 = 0.11, sigma2 = 0.16, rho = 0.63)
   TwoRiskyAssetsOption(TypeFlag = "pmax", S1 = 100, S2 = 105, 
     X = 98, Time = 0.5, r = 0.05, b1 = -0.01, b2 = -0.04, 
     sigma1 = 0.11, sigma2 = 0.16, rho = 0.63)

## Spread-Option Approximation [2.8.5]:
   xmpOptions("\nNext: Spread-Option Approximation > ")
   SpreadApproxOption(TypeFlag = "c", S1 = 28, S2 = 20, X = 7, 
     Time = 0.25, r = 0.05, sigma1 = 0.29, sigma2 = 0.36, rho = 0.42)
	
}


\keyword{math}


\eof
\name{LookbackOptions}

\alias{LookbackOptions}

\alias{FloatingStrikeLookbackOption}
\alias{FixedStrikeLookbackOption}
\alias{PTFloatingStrikeLookbackOption}
\alias{PTFixedStrikeLookbackOption}
\alias{ExtremeSpreadOption}


\title{Valuation of Lookback Options}


\description{
  
	This is a collection of functions to valuate lookback options.
  	The payoff from a pathdependent lookback call (put) depends on the 
  	exercise price being set to the minimum (maximum) asset price achieved 
  	during the life of the option. Thus, a lookback call (put) allows 
  	the purchaser to buy (sell) the asset at its minimum (maximum) price.
	\cr
	
	The functions are:

	\tabular{rll}{
		1 	\tab 	\code{FloatingStrikeLookbackOption} 	
	 		\tab 	Floating Strike Lookback Option, \cr
	 	2 	\tab 	\code{FixedStrikeLookbackOption} 		
	 		\tab 	Fixed Strike Lookback Option, \cr
	 	3 	\tab 	\code{PTFloatingStrikeLookbackOption} 	
	 		\tab 	PT Floating Strike Lookback Option, \cr
	 	4 	\tab 	\code{PTFixedStrikeLookbackOption} 		
	 		\tab 	PT Fixed Strike Lookback Option, \cr
	 	5	\tab	\code{ExtremeSpreadOption}
	 		\tab 	Extreme Spread Options.}
  	
}


\usage{
FloatingStrikeLookbackOption(TypeFlag, S, SMinOrMax, Time, r, b, sigma)
FixedStrikeLookbackOption(TypeFlag, S, SMinOrMax, X, Time, r, b, sigma)
PTFloatingStrikeLookbackOption(TypeFlag, S, SMinOrMax, time1, Time2, r, 
	b, sigma, lambda) 
PTFixedStrikeLookbackOption(TypeFlag, S, X, time1, Time2, r, b, sigma)
ExtremeSpreadOption(TypeFlag, S, SMin, SMax, time1, Time2, r, b, sigma)
}


\arguments{

	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{lambda}{
	    The \code{lambda} factor enables the creation of so-called
	    "fractional" lookback options where the strike is fixed at
	    some percentage or below the extremum, i.e. \code{lambda} 
	    is greater than 1 for calls, and between 0 and 1 for puts.
	    }
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
	\item{S}{
		the asset price, a numeric value.
		}
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{SMax, SMin}{
		[ExtremeSpread*] -
		the maximum (minimum) value of the underlying asset. Note, the
		payoff at maturity of the extreme spread call (put) equals the 
		positive part of the difference between the maximum (minimum)
		value of the underlying asset, \code{SMax}, of the second
		(first) period and the maximum (minimum) of the underlying
		asset of the first (second) period. Likewise, reverse conditions
		are valid for the reverse extreme spread option.
		}
	\item{SMinOrMax}{
		the lowest price observed of the underlying in the case of
		the coll, or the highest price in the case of the put. A 
		numeric value.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{time1, Time2}{
		[PTFloatingStrikeLookback*] -
		the time to the end of the lookback period \code{time1}, and
		the time to expiry \code{Time2} where \code{time1<Time2}, \cr
		[PTFixedStrikeLookback*] -
		the predetermined time \code{time1} where the lookback 
		period starts, and the time to expiry \code{Time2}, \cr
		[ExtremeSpread*] -
		the two time periods, one starting today and ending at 
		\code{time1}, and the other starting at \code{time1} and
		ending at the maturity time \code{Time2} of the option.
		}
	\item{TypeFlag}{
		usually a character string either \code{"c"} for a call option 
		or a \code{"p"} for a put option, except for \cr
		[ExtremeSpread*] -
		a character string either, \cr
		\code{"c"} for the extreme call, \cr
		\code{"p"} for the extreme put, \cr
		\code{"cr"} for the reverse extreme call, \cr
		\code{"pr"} for the revers extreme put.
		}
	\item{X}{
		the exercise price, a numeric value.
		}

}


\details{
  
  	\bold{Floating Strike Lookback Options:}
	\cr\cr
  	The lookback call (put) option gives the holder the right to buy (sell) 
  	an asset at its lowest (highest) price observed during the life of the 
  	option. This observed price is applied as the strike price. The payout 
  	for a call option is essentially the asset price minus the minimum spot 
  	price observed during the life of the option. The payout for a put option 
  	is essentially the maximum spot price observed during the life of the 
  	option minus the asset price. Therefore, a floating strike lookback 
  	option is always in the money and should always be exercised. Floating 
  	strike options can be priced analytically using a model introduced by 
  	Goldman, Sosin, and Gatto (1979). Monte Carlo simulation is used for 
  	the numerical calculation of a European style floating strike options.
  	\cr
  	[Haug's Book, Chapter 2.9.1]
  	\cr
  
  	
  	\bold{Fixed Strike Lookback Options:}
  	\cr\cr
	For a fixed strike lookback option, the strike price is known in advance. 
	The call option payoff is given by the difference between the maximum 
	observed price of the underlying asset during the life of the option and 
	the fixed strike price. The put option payoff is given by the difference 
	between the fixed strike price and the minimum observed price of the 
	underlying asset during the life of the option. A fixed strike lookback 
	call (put) option payoff is equal to that of a standard plain call (put) 
	option when the final asset price is the maximum (minimum) observed value 
	during the options life. Fixed strike lookback options can be priced 
	analytically using a model introduced by Conze and Viswanathan (1991).
	\cr
	[Haug's Book, Chapter 2.9.2]
	\cr
  
	
  	\bold{Partial-Time Floating Strike Options:}
  	\cr\cr
  	For a partial-time floating strike lookback option, the lookback period 
  	starts at time zero and ends at an arbitrary date before expiration. 
  	Except for the partial lookback period, the option is similar to a 
  	floating strike lookback option. The partial-time floating strike 
  	lookback option is cheaper than a similar standard floating strike 
  	lookback option. Partial-time floating strike lookback options can be 
  	priced analytically using a model introduced by Heynen and Kat (1994).
  	\cr
  	[Haug's Book, Chapter 2.9.3]
  	\cr
  
  	
  	\bold{Partial-Time Fixed Strike Options:}
  	\cr\cr
  	For a partial-time fixed strike lookback option, the lookback period 
  	starts at a predetermined date after the initialization date of the 
  	option. The partial-time fixed strike lookback call option payoff is 
  	given by the difference between the maximum observed price of the 
  	underlying asset during the lookback period and the fixed strike 
  	price. The partial-time fixed strike lookback put option payoff is 
  	given by the difference between the fixed strike price and the 
  	minimum observed price of the underlying asset during the lookback 
  	period. The partial-time fixed strike lookback option is cheaper than 
  	a similar standard fixed strike lookback option. Partial-time fixed 
  	strike lookback options can be priced analytically using a model 
  	introduced by Heynen and Kat (1994).
  	\cr
  	[Haug's Book, Chapter 2.9.4]
  	\cr
  
  	
  	\bold{Extreme Spread Options:}
  	\cr\cr
  	The time to maturity of an extreme spread option is divided into two 
  	periods: one period starting at time zero and ending at some arbitrary 
  	date, and another starting at that arbitrary date and ending at the 
  	expiration date. A payoff at maturity of an extreme spread call (put) 
  	option equals the positive part of the difference between the maximum 
  	(minimum) value of the underlying asset of the second (first) period 
  	and the maximum (minimum) value of the underlying asset of the first 
  	(second) period.[1] The payoff at expiration of a reverse extreme 
  	spread call (put) option equals the positive part of the difference 
  	between the minimum (maximum) of the underlying asset of the second 
  	(first) period and the minimum (maximum) value of the underlying asset 
  	of the first (second) period. Extreme spread options can be priced 
  	analytically using a model introduced by Bermin (1996).
  	\cr
  	[Haug's Book, Chapter 2.9.5]

}


\note{
  
	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 2.9 of Haug's Book (1997).

}



\value{

	The option price, a numeric value.
	
}



\examples{
## Examples from Chapter 2.9 in E.G. Haug's Option Guide (1997)

## Floating Strike Lookback Option [2.9.1]:
   xmpOptions("\nStart: Floating Strike Lookback Option > ")
   FloatingStrikeLookbackOption(TypeFlag = "c", S = 120, 
     SMinOrMax = 100, Time = 0.5, r = 0.10, b = 0.10-0.06, 
     sigma = 0.30)
  
## Fixed Strike Lookback Option [2.9.2]:
   xmpOptions("\nNext: Fixed Strike Lookback Option > ")
   FixedStrikeLookbackOption(TypeFlag = "c", S = 100, 
     SMinOrMax = 100, X = 105, Time = 0.5, r = 0.10, b = 0.10, 
     sigma = 0.30)
   
## Partial Time Floating Strike Lookback Option [2.9.3]:
   xmpOptions("\nNext: PT Floating Strike Option > ")
   PTFloatingStrikeLookbackOption(TypeFlag = "p", S = 90, 
     SMinOrMax = 90, time1 = 0.5, Time2 = 1, r = 0.06, b = 0.06, 
     sigma = 0.20, lambda  = 1)
   
## Partial Time Fixed Strike Lookback Option [2.9.4]:
   xmpOptions("\nNext: PT Fixed Strike Lookback Option > ")
   PTFixedStrikeLookbackOption(TypeFlag = "c", S = 100, X = 90, 
	 time1 = 0.5, Time2 = 1, r = 0.06, b = 0.06, sigma = 0.20)
	 
## Extreme Spread Option [2.9.5]:
   xmpOptions("\nNext: PT Fixed Strike Lookback Option > ")
   ExtremeSpreadOption(TypeFlag = "c", S = 100, SMin = NA, 
     SMax = 110, time1 = 0.5, Time2 = 1, r = 0.1, b = 0.1, 
     sigma = 0.30)
   ExtremeSpreadOption(TypeFlag = "cr", S = 100, SMin = 90, 
     SMax = NA, time1 = 0.5, Time2 = 1, r = 0.1, b = 0.1, 
     sigma = 0.30)
		
}


\references{
Bermin H.P. (1996b);
	\emph{Exotic Lookback Options: The case of Extreme Spread
		Options},
	Department of Economics, Lund University, Sweden.

Conze A., Viswanathan R. (1991);
	\emph{Path Dependent Options: The Case of Lookback Options},
	Journal of Finance 46, 1893--1907.

Goldmann B.M., Sosin H.B., Gatto M.A. (1993);
	\emph{Path Dependent Options: Buy at the Low, Sell at the High},
	Journal of Finance 34, 1111.

Haug E.G. (1997); 
	\emph{The Complete Guide to Option Pricing Formulas}, 
	McGraw-Hill, New York.
	
Heynen R.C., Kat H.M. (1994);
	\emph{Selective Memory},
	Risk Magazine 7, 1994.
}


\author{

	Diethelm Wuertz for this R-Port.

}


\keyword{math}


\eof
\name{BarrierOptions}

\alias{BarrierOptions}

\alias{StandardBarrierOption}
\alias{DoubleBarrierOption}
\alias{PTSingleAssetBarrierOption}
\alias{TwoAssetBarrierOption}
\alias{PTTwoAssetBarrierOption}
\alias{LookBarrierOption}
\alias{DiscreteBarrierOption}
\alias{SoftBarrierOption}


\title{Valuation of Barrier Options}


\description{
  
	This is a collection of functions to valuate barrier options.  
  	Barrier options are path-dependent options, with payoffs that depend on 
  	the price of the underlying asset at expiration and whether or not the 
  	asset price crosses a barrier during the life of the option. There are 
  	two categories or types of Barrier options: "knock-in" and "knock-out". 
  	"Knock-in" or "in" options are paid for up front, but you do not receive 
  	the option until the asset price crosses the barrier. "Knock-out" or "out" 
  	options come into existence on the issue date but becomes worthless if the 
  	asset price hits the barrier before the expiration date. If the option is a 
  	knock-in (knock-out), a predetermined cash rebate may be paid at expiration 
  	if the option has not been knocked in (knocked-out) during its lifetime. 
  	The barrier monitoring frequency specifies how often the price is checked 
  	for a breach of the barrier. All of the analytical models have a flag to 
  	change the monitoring frequency where the default frequency is continuous.	
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{StandardBarrierOption} 	  
	 		\tab 	Standard Barrier Option, \cr
	 	2 	\tab 	\code{DoubleBarrierOption} 		  
	 		\tab 	Double Barrier Option, \cr
	 	3 	\tab 	\code{PTSingleAssetBarrierOption} 
	 		\tab 	Partial Time Single Asset Barrier Option, \cr
	 	4 	\tab 	\code{TwoAssetBarrierOption} 	  
	 		\tab 	Two Asset Barrier Option, \cr
	 	5 	\tab 	\code{PTTwoAssetBarrierOption} 	  
	 		\tab 	Partial Time Two Asset Barrier Option, \cr
	 	6 	\tab 	\code{LookBarrierOption} 		  
	 		\tab 	Lookback Barrier Option, \cr
	 	7 	\tab 	\code{DiscreteBarrierOption} 	  
	 		\tab 	Discrete Barrier Option \cr
	 	8 	\tab 	\code{SoftBarrierOption} 		  
	 		\tab 	Soft Barrier Option. }
  	
}


\usage{
StandardBarrierOption(TypeFlag, S, X, H, K, Time, r, b, sigma)
DoubleBarrierOption(TypeFlag, S, X, L, U, Time, r, b, sigma, delta1, delta2)
PTSingleAssetBarrierOption(TypeFlag, S, X, H, time1, Time2, r, b, sigma)
TwoAssetBarrierOption(TypeFlag, S1, S2, X, H, Time, r, b1, 
	b2, sigma1, sigma2, rho)
PTTwoAssetBarrierOption(TypeFlag, S1, S2, X, H, time1, Time2, r, b1, b2, 
	sigma1, sigma2, rho)
LookBarrierOption(TypeFlag, S, X, H, time1, Time2, r, b, sigma)
DiscreteBarrierOption(S, H, sigma, dt)
SoftBarrierOption(TypeFlag, S, X, L, U, Time, r, b, sigma)
}


\arguments{

	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{b1, b2}{
		[PTTwoAssetBarrier*] -
		the annualized cost-of-carry rate for the first and second
		asset, a numeric value.
		}
	\item{delta1, delta2}{
		[DoubleBarrier*] - 
		numeric values which determine the curvature of the lower 
		\code{L} and upper \code{U} bounds. The case of \cr
		\code{delta1=delta2=0} corresponds to two flat boundaries, \cr
		\code{delta1<0<delta2} correponds to a lower boundary exponentially
		growing as time elapses, while the upper boundary will be 
		exponentially decaying, \cr
		\code{delta1>0>delta2} correponds to a convex downward lower 
		boundary and a convex upward upper boundary.
		}
	\item{dt}{
		[DiscreteBarrier*] -
		time between monitoring instants, a numeric value.
		}
	\item{H}{
		[StandardBarrier*] -
		the barrier value, a numeric value.
		}	 
	\item{K}{
		[StandardBarrier*] - 
		for an "In"-Barrier a prespecified cash rebate which is paid 
		out at option expiration if the option has not been knocked 
		in during its lifetime,\cr
		for an "Out"-Barrier a prespecified cash rebate which is paid 
		out at option expiration if the option has not been knocked 
		out before its lifetime,
		a numerical value.
		}
	\item{L, U}{
		[DoubleBarrier*] - 
		the lower and upper boundary to be touched, numerical values.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
	\item{rho}{
		[TwoAssetBarrier*] -
		the correlation of the volatility between the first and
		second asset, a numeric value.
		}
	\item{S}{
		the asset price, a numeric value.
		}
	\item{S1, S2}{
		[PTTwoAssetBarrier*] -
		the price of the first and second asset, a numeric value.
		}	
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{sigma1, sigma2}{
		[PTTwoAssetBarrier*] -
		the annualized volatility of the first and second underlying 
		security, numeric values.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{time1, Time2}{
		[PTSingleAssetBarrier*][PTTwoAssetBarrier*] -
		so called type "A" options (see the \code{TypeFlag} argument)
		will have the location of the monitoring period starting at 
		the options starting date and ending at an arbitrary time 
		\code{time1} before expiration time \code{Time2}. 
		Partial-time-end-barrier options will have the location of 
		the monitoring period starting at an arbitrary time \code{time1} 
		before expiration time \code{Time2}, and ending at expiration 
		time.\cr
		[LookBarrier*] -
		the lookbarrier option's barrier monitoring period starts at 
		the options starting date and ends at an arbitrary time 
		\code{time1} before expiration time \code{Time2}.
		}
	\item{TypeFlag}{
		usually a character string either \code{"c"} for a call option 
		or a \code{"p"} for a put option.\cr\cr\code{ }
		[StandardBarrier*] -
		here\cr 
		\code{"cdi"} denotes a down-and-in call,\cr 
  		\code{"cui"} denotes an up-and-in call,\cr
  		\code{"cdo"} denotes a down-and-out call, and\cr
  		\code{"cuo"} denotes an up-and-out call.\cr 
  		Similarily, the type flags for the corresponding puts are
  		\code{"pdi"}, \code{"pui"}, \code{"pdo"}, and \code{"puo"}.\cr\cr
  		[DoubleBarrier*] -
  		here\cr 
  		\code{"co"} denotes an up-and-out-down-and-out call,\cr
    	\code{"ci"} denotes an up-and-in-down-and-in call,\cr
    	\code{"po"} denotes an up-and-out-down-and-out put, and\cr
    	\code{"pi"} denotes an up-and-in-down-and-in call.\cr\cr
    	[PTSingleAssetBarrier*] -
    	here\cr
    	\code{"cdoA"} denotes a down-and-out call of type "A",\cr
    	\code{"cuoA"} denotes an up-and-out call of type "A",\cr
    	\code{"pdoA"} denotes a down-and-out put of type "A",\cr
    	\code{"puoA"} denotes an up-and-out put of type "A",\cr
    	\code{"coB1"} denotes an out-call of type "B1",\cr
    	\code{"poB1"} denotes an out-call of type "B1",\cr
    	\code{"cdoB2"} denotes a down-and-out call of type "B2",\cr
    	\code{"cuoB2"} denotes an up-and-out call of type "B2".\cr	
    	Note, a partial-time-start-barrier option is called a type "A"
    	option, a partial-time-end-out-call is a called a type "B"
    	option. There are two types of "B" options: "B1" is defined
    	such that only a barrier hit or crossed causes the option to be
    	knocked out, and a "B2" is defined such that a down-and-out-call
    	is knocked out as soon as the underlying price is below the 
    	barrier.\cr\cr
    	[TwoAssetBarrier*][PTTwoAssetBarrier*] - 
		here\cr
		\code{"cuo"} denotes an up-and-out call,\cr
		\code{"cui"} denotes an up-and-in call,\cr
		\code{"cdo"} denotes a down-and-out call,\cr
		\code{"cdi"} denotes a down-and-in call,\cr
		\code{"puo"} denotes an up-and-out put,\cr
		\code{"pui"} denotes an up-and-in put,\cr
		\code{"pdo"} denotes a down-and-out put,\cr
		\code{"pdi"} denotes a down-and-in put.\cr\cr
		[LookBarrier*][SoftBarrier*] -
		here\cr
		\code{"cuo"} denotes an up-and-out call,\cr
		\code{"cui"} denotes an up-and-in call,\cr
		\code{"pdo"} denotes a down-and-out put,\cr
		\code{"pdi"} denotes a down-and-in put.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
		
}


\details{

  	\bold{Single [Standard] Barrier Options:}
	\cr\cr
  	There are four types of single barrier options. The type flag 
  	\code{"cdi"} denotes a down-and-in call,
  	\code{"cui"} denotes an up-and-in call,
  	\code{"cdo"} denotes a down-and-out call, and
  	\code{"cuo"} denotes an up-and-out call. 
  	Similarily, the type flags for the corresponding puts are
  	\code{cdi}, \code{cui}, \code{cdo}, and \code{cuo}.
  	A down-and-in option comes into existence 
  	and knocked-in only if the asset price falls to the barrier level. An 
  	up-and- in option comes into existence and knocked-in only if the asset 
  	price rises to the barrier level. A down-and-out option comes into 
  	existence and knocked-out only if the asset price falls to the barrier 
  	level. An up-and-in option comes into existence and knocked-out only if 
  	the asset price rises to the barrier level. European single barrier 
  	options can be priced analytically using a model introduced by Reiner 
  	and Rubinstein (1991). A trinomial lattice is used for the numerical 
  	calculation of an American or European style single barrier options.
  	\cr
  	[Haug's Book, Chapter 2.10.1]
  	\cr
 
  	
  	\bold{Double Barrier Options:}
	\cr\cr
  	A double barrier option is either knocked in or knocked out if the 
  	asset price touches the lower or upper barrier during its lifetime. 
    The type flag
    \code{"co"} denotes an up-and-out-down-and-out call,
    \code{"ci"} denotes an up-and-in-down-and-in call,
    \code{"po"} denotes an up-and-out-down-and-out put, and
    \code{"pi"} denotes an up-and-in-down-and-in call.
    Once a barrier is crossed, the option comes into existence if it is 
    a knock-in barrier or becomes worthless if it is a knocked out barrier. 
    Double barrier options can be priced analytically using a model 
    introduced by Ikeda and Kunitomo (1992).
  	\cr
  	[Haug's Book, Chapter 2.10.2]
  	\cr
 
  	
  	\bold{Partial-Time Barrier Options:}
	\cr\cr
  	For single asset partial-time barrier options, the monitoring period 
  	for a barrier crossing is confined to only a fraction of the options 
  	lifetime. There are two types of partial-time barrier options: 
  	partial-time-start and partial-time-end. Partial-time-start barrier 
  	options have the monitoring period start at time zero and end at an 
  	arbitrary date before expiration. Partial-time-end barrier options 
  	have the monitoring period start at an arbitrary date before expiration 
  	and end at expiration. Partial-time-end barrier options are then broken 
  	down again into two categories: B1 and B2. Type B1 is defined such that 
  	only a barrier hit or crossed causes the option to be knocked out. 
  	There is no difference between up and down options. Type B2 options 
  	are defined such that a down-and-out call is knocked out as soon as 
  	the underlying price is below the barrier. Similarly, an up-and-out 
  	call is knocked out as soon as the underlying price is above the barrier.
  	Partial-time barrier options can be priced analytically using a model 
  	introduced by Heynen and Kat (1994).
  	\cr
  	[Haug's Book, Chapter 2.10.3]
  	\cr
  	
  	
  	\bold{Two-Asset Barrier Options:}
	\cr\cr
  	The underlying asset, Asset 1, determines how much the option is in 
  	or out-of-the-money. The other asset, Asset 2, is the trigger asset 
  	that is linked to barrier hits. Two-asset barrier options can be 
  	priced analytically using a model introduced by Heynen and Kat (1994).
  	\cr
  	[Haug's Book, Chapter 2.10.4]
  	\cr
  	
  	
  	\bold{Lookback Barrier Options:}
	\cr\cr
  	A look-barrier option is the combination of a forward starting fixed 
  	strike Lookback option and a partial time barrier option. The options 
  	barrier monitoring period starts at time zero and ends at an arbitrary 
  	date before expiration. If the barrier is not triggered during this 
  	period, the fixed strike Lookback option will be kick off at the end 
  	of the barrier tenor. Lookback barrier options can be priced analytically 
  	using a model introduced by Bermin (1996).
  	\cr
  	[Haug's Book, Chapter 2.10.6]
  	\cr
 
  	
  	\bold{Partial-Time-Two-Asset Options:}
	\cr\cr
  	Partial-time two-asset barrier options are similar to standard two-asset 
  	barrier options, except that the barrier hits are monitored only for a 
  	fraction of the option's lifetime. The option is knocked in or knocked 
  	out is Asset 2 hits the barrier during the monitoring period. The payoff 
  	depends on Asset 1 and the strike price. Partial-time two-asset barrier 
  	options can be priced analytically using a model introduced by Bermin 
  	(1996).
  	\cr
  	[Haug's Book, Chapter 2.10.5]
  	\cr
  	
  	
  	\bold{Soft Barrier Options:}
	\cr\cr
  	A soft-barrier option is similar to a standard barrier option, except 
  	that the barrier is no longer a single level. Rather, it is a soft 
  	range between a lower level and an upper level. Soft-barrier options 
  	are knocked in or knocked out proportionally. Introduced by Hart and 
  	Ross (1994), the valuation formula can be used to price soft-down-and-in 
  	call and soft-up-and-in put options. The value of the related "out" 
  	option can be determined by subtracting the "in" option value from the 
  	value of a standard plain option. Soft-barrier options can be priced 
  	analytically using a model introduced by Hart and Ross (1994).
  	\cr
  	[Haug's Book, Chapter 2.10.8]

}


\note{

  	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 2.10 of Haug's Book (1997).
  	
}


\value{

	The option price, a numeric value.
	
}



\examples{
## Examples from Chapter 2.10 in E.G. Haug's Option Guide (1997)

## Standard Barrier Option [2.10.1]:
   xmpOptions("\nStart: Down-and-Out Barrier Option > ")
   # down-and-out Barrier Call
   StandardBarrierOption(TypeFlag = "cdo", S = 100, X = 90, 
     H = 95, K = 3, Time = 0.5, r = 0.08, b = 0.04, sigma = 0.25)
   
## Double Barrier Option [2.10.2]:
   xmpOptions("\nNext: Double Barrier Option > ")
   DoubleBarrierOption(TypeFlag = "co", S = 100, X = 100, L = 50, 
     U = 150, Time = 0.25, r = 0.10, b = 0.10, sigma = 0.15, 
     delta1 = -0.1, delta2 = 0.1)
   
## Partial Time Single-Asset Barrier Option [2.10.3]:
   xmpOptions("\nNext: PT Single-Asset Barrier Option > ")
   PTSingleAssetBarrierOption(TypeFlag = "coB1", S = 95, X = 110, 
     H = 100, time1 = 0.5, Time2 = 1, r = 0.20, b = 0.20, 
     sigma = 0.25)
   
## Two Asset Barrier Option [2.10.4]:
   xmpOptions("\nNext: Two Asset Barrier Option > ")
   TwoAssetBarrierOption(TypeFlag = "puo", S1 = 100, S2 = 100, 
     X = 110, H = 105, Time = 0.5, r = 0.08, b1 = 0.08, b2 = 0.08, 
     sigma1 = 0.2, sigma2 = 0.2, rho = -0.5)
   
## PT Two Asset Barrier Option [2.10.5]:
   xmpOptions("\nNext: PT Two Asset Barrier Option > ")
   PTTwoAssetBarrierOption(TypeFlag = "pdo", S1 = 100, S2 = 100, 
     X = 100, H = 85, time1 = 0.5, Time2 = 1, r = 0.1, b1 = 0.1, 
     b2 = 0.1, sigma1 = 0.25, sigma2 = 0.30, rho = -0.5)
   
## Look Barrier Option [2.10.6]:
   xmpOptions("\nNext: Look Barrier Option > ")
   LookBarrierOption(TypeFlag = "cuo", S = 100, X = 100, H = 130, 
     time1 = 0.25, Time2 = 1, r = 0.1, b = 0.1, sigma = 0.15)
   LookBarrierOption(TypeFlag = "cuo", S = 100, X = 100, H = 110, 
     time1 = 1, Time2 = 1, r = 0.1, b = 0.1, sigma = 0.30)
   
## Discrete Barrier Option [2.10.7]:  
   xmpOptions("\nNext: Discrete Barrier Option > ")
   DiscreteBarrierOption(S = 100, H = 105, sigma = 0.25, dt = 0.1)
   
## Soft Barrier Option [2.10.8]:
   xmpOptions("\nNext: Soft Barrier Option > ")
   SoftBarrierOption(TypeFlag = "cdo", S = 100, X = 100, L = 70, 
     U = 95, Time = 0.5, r = 0.1, b = 0.05, sigma = 0.20)
  
}


\references{
Haug E.G. (1997); 
	\emph{The complete Guide to Option Pricing Formulas}, 
	Chapter 2.10, McGraw-Hill, New York.
}


\author{

 	Diethelm Wuertz for this R-Port.
 
}


\keyword{math}


\eof
\name{BinaryOptions}

\alias{BinaryOptions}

\alias{GapOption}
\alias{CashOrNothingOption}
\alias{TwoAssetCashOrNothingOption}
\alias{AssetOrNothingOption}
\alias{SuperShareOption}
\alias{BinaryBarrierOption}


\title{Valuation of Binary Options}


\description{
  
	This is a collection of functions to valuate binary options. 
  	Binary options, also known as digital options, have discontinuous 
  	payoffs. They can be used as building blocks to develop options 
  	with more complicated payoffs. For example, a regular European 
  	call option is equivalent to a long position in an asset-or-nothing 
  	call and a short position in a cash-or-nothing call, where the both 
  	options have the same strike price and the cash payoff of the 
  	cash-or-nothing option equals the strike price. Unlike standard 
  	European style options, the payout for binary options does not 
  	depend on how much it is in-the-money but rather whether or not 
  	it is on the money. The options payoff is fixed at the options 
  	inception and is based on the price of the underlying asset on 
  	the expiration date. Binary options may also incorporate barriers, 
  	as is the case with binary-barrier options.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{GapOption} 					
	 		\tab 	Gap Option, \cr
	 	2 	\tab 	\code{CashOrNothingOption} 			
	 		\tab 	Cash Or Nothing Option, \cr
	 	3 	\tab 	\code{TwoAssetCashOrNothingOption} 	
	 		\tab 	Two Asset Cash Or Nothing Option, \cr
	 	4 	\tab 	\code{AssetOrNothingOption} 		
	 		\tab	Asset Or Nothing Option, \cr
	 	5 	\tab 	\code{SuperShareOption} 				
	 		\tab 	Super Share Option, \cr
	 	6 	\tab 	\code{BinaryBarrierOption} 			
	 		\tab 	Binary Barrier Option. }

}


\usage{
GapOption(TypeFlag, S, X1, X2, Time, r, b, sigma)
CashOrNothingOption(TypeFlag, S, X, K, Time, r, b, sigma) 
TwoAssetCashOrNothingOption(TypeFlag, S1, S2, X1, X2, K, Time, 
	r, b1, b2, sigma1, sigma2, rho)
AssetOrNothingOption(TypeFlag, S, X, Time, r, b, sigma)
SuperShareOption(S, XL, XH, Time, r, b, sigma)
BinaryBarrierOption(TypeFlag, S, X, H, K, Time, r, b, sigma, eta, phi)
}


\arguments{

	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{b1, b2}{
		[TwoAssetCashOrNothing*] -
		the annualized cost-of-carry rate for the first and second
		asset, a numeric value.
		}
	\item{eta, phi}{
		[BinaryBarrier*] -
		a set of parameters to price 28 different types of Binary 
		Barrier options:\cr
		01: \code{eta=+1, phi=NA, [S>H]} down-and-in cash-at-hit-or-nothing, \cr
		02: \code{eta=-1, phi=NA, [S<H]} up-and-in cash-at-hit-or-nothing, \cr
		03: \code{eta=+1, phi=NA, [S>H]} down-and-in asset-at-hit-or-nothing, \cr
		04: \code{eta=-1, phi=NA, [S<H]} up-and-in asset-at-hit-or-nothing, \cr
		05: \code{eta=+1, phi=-1, [S>H]} down-and-in cash-at-expiry-or-nothing, \cr
		06: \code{eta=-1, phi=+1, [S<H]} up-and-in cash-at-expiry-or-nothing, \cr
		07: \code{eta=+1, phi=-1, [S>H]} down-and-in asset-at-expiry-or-nothing, \cr
		08: \code{eta=-1, phi=+1, [S<H]} up-and-in asset-at-expiry-or-nothing, \cr
		09: \code{eta=+1, phi=+1, [S>H]} down-and-out cash-or-nothing, \cr
		10: \code{eta=-1, phi=-1, [S<H]} up-and-out cash-or-nothing, \cr
		11: \code{eta=+1, phi=+1, [S>H]} down-and-out asset-or-nothing, \cr
		12: \code{eta=-1, phi=-1, [S<H]} up-and-out asset-or-nothing, \cr
		13: \code{eta=+1, phi=+1, [S>H]} down-and-in cash-or-nothing call, \cr
		14: \code{eta=-1, phi=+1, [S<H]} up-and-in cash-or-nothing call, \cr
		15: \code{eta=+1, phi=+1, [S>H]} down-and-in asset-or-nothing call, \cr
		16: \code{eta=-1, phi=+1, [S<H]} up-and-in asset-or-nothing call, \cr
		17: \code{eta=+1, phi=-1, [S>H]} down-and-in cash-or-nothing put, \cr
		18: \code{eta=-1, phi=-1, [S<H]} up-and-in cash-or-nothing put, \cr
		19: \code{eta=+1, phi=-1, [S>H]} down-and-in asset-or-nothing put, \cr
		20: \code{eta=-1, phi=-1, [S<H]} up-and-in asset-or-nothing put, \cr
		21: \code{eta=+1, phi=+1, [S>H]} down-and-out cash-or-nothing call, \cr
		22: \code{eta=-1, phi=+1, [S<H]} up-and-out cash-or-nothing call, \cr
		23: \code{eta=+1, phi=+1, [S>H]} down-and-out asset-or-nothing call, \cr
		24: \code{eta=-1, phi=-1, [S<H]} up-and-out asset-or-nothing call, \cr
		25: \code{eta=+1, phi=-1, [S>H]} down-and-out cash-or-nothing put, \cr
		26: \code{eta=-1, phi=-1, [S<H]} up-and-out cash-or-nothing put, \cr
		27: \code{eta=+1, phi=-1, [S>H]} down-and-out asset-or-nothing put, \cr
		28: \code{eta=-1, phi=-1, [S<H]} up-and-out asset-or-nothing put.
		}
	\item{H}{
		[BinaryBarrier*] -
		the barrier value, a numeric value.
		}	 
	\item{K}{
		[CashOrNothing*] - 
		the cash amount at expiry if the option is in the money, 
		a numerical value.\cr
		[TwoAssetCashOrNothing*] -
		for the cash-or-nothing call the cash amount at expiry if
		asset \code{S1} is above the strike \code{X1} and asset 
		\code{S2} is above strike \code{X2} at expiration, 
		\cr
		for the cash-or-nothing put the cash amount at expiry if
		asset \code{S1} is below the strike \code{X1} and asset 
		\code{S2} is below strike \code{X2} at expiration, 
		\cr
		for the cash-or-nothing up-down the cash amount at expiry if
		asset \code{S1} is above the strike \code{X1} and asset 
		\code{S2} is below strike \code{X2} at expiration, 
		\cr
		for the cash-or-nothing down-up the cash amount at expiry if
		asset \code{S1} is below the strike \code{X1} and asset 
		\code{S2} is above strike \code{X2} at expiration.\cr
		[BinaryBarrier*] -
		the prespecified cash amount, a numeric value.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
	\item{rho}{
		[TwoAssetCashOrNothing*] -
		the correlation of the volatility between the first and
		second asset, a numeric value.
		}
	\item{S}{
		the asset price, a numeric value.
		}
	\item{S1, S2}{
		[TwoAssetCashOrNothing*] -
		the price of the first and second asset, a numeric value.
		}		
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{sigma1, sigma2}{
		[TwoAssetCashOrNothing*] -
		the annualized volatility of the first and second underlying 
		security, numeric values.
		}
	\item{Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{TypeFlag}{
		a character string either \code{"c"} for a call option or 
		a \code{"p"} for a put option. \cr
		[TwoAssetCashOrNothing*] -
		a character string either \code{"c"} for a call option, or 
		a \code{"p"} for a put option, or a \code{"ud"} for an 
		up-down option, or a \code{"du"} for a down-up option.\cr
		[BinaryBarrier*] -
		an integer between 1 and 28, selecting one of the 28 types,
		for a definition lookup the arguments \code{eta} and 
		\code{phi}.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
	\item{X1, X2}{
		[GapOption][TwoAssetCashOrNothing*]  -
		the first and the second exercise price, a numeric value.
		}
	\item{XL, XH}{
		[SuperShare*]  -
		the lower and upper boundary strike, a numeric value.
		}			

}


\details{

  	\bold{Gap Options:}
	\cr\cr
  	The payoff on a gap option depends on the usual factors of a plain option, 
  	but is also affected by a "gap"  amount of exercise prices, which may be 
  	positive or negative. Note, that a gap call (put) option is equivalent to 
  	being long (short) an asset-or-nothing call (put) and short (long) a 
  	cash-or-nothing call (put). The option price is calculated analytically 
  	according to Reiner and Rubinstein (1991).
  	\cr
  	[Haug's Book, Chapter 2.11.1]
  	\cr
  	

  	\bold{Cash-or-Nothing Options:}
	\cr\cr
  	For this option a predetermined amount is paid at expiration if the 
  	asset is above for a call or below for a put some strike level. The 
  	amount independent of the path taken. These options require no payment 
  	of an exercise price. The exercise price determines whether or not the 
  	option returns a payoff. The value of a cash-or-nothing call (put) 
  	option is the present value of the fixed cash payoff multiplied by 
  	the probability that the terminal price will be greater than (less than) 
  	the exercise price. The option price is calculated analytically 
  	according to Reiner and Rubinstein (1991).
  	\cr
  	[Haug's Book, Chapter 2.11.2]
  	\cr
  	
 
  	\bold{Two-Asset-Cash-Or-Nothing Options:}
	\cr\cr
  	These options are building blocks for constructing more complex exotic 
  	options. There are four types of two-asset cash-or-nothing options, the 
  	first two situationsa are: A two-asset-cash-or-nothing call pays out a 
  	fixed cash amount if the price of the first asset is above (below) the 
  	strike price of the first asset and the price of the second asset is also 
  	above (below) the strike price of the second asset at expiration. The 
  	other two situations arise under the following conditions: A two-asset 
  	cash-or-nothing down-up pays out a fixed cash amount if the price of the 
  	first asset is is below (above) the strike price of the first asset and 
  	the price of the second asset is above (below) the strike price of the 
  	second asset at expiration. The option price is calculated analytically 
  	according to Heynen and Kat (1996).
  	\cr
  	[Haug's Book, Chapter 2.11.3]
  	\cr
  	

  	\bold{Asset-Or-Nothing Options:}
	\cr\cr
  	In this option a predetermined asset value is paid if the asset is, at 
  	expiration, above for a call or below for a put some strike level, 
  	independent of the path taken. For a call (put) the terminal price is 
  	greater than (less than) the exercise price, the call (put) expires 
  	worthless. The exercise price is never paid. Instead, the value of the 
  	asset relative to the exercise price determines whether or not the 
  	option returns a payoff. The value of an asset-or-nothing call (put) 
  	option is the present value of the asset multiplied by the probability 
  	that the terminal price will be greater than (less than) the exercise 
  	price. The option price is calculated analytically according to Cox 
  	and Rubinstein (1985).
  	\cr
  	[Haug's Book, Chapter 2.11.4]
  	\cr

  	
  	\bold{Supershare Options:}
	\cr\cr
  	These options represents a contingent claim on a fraction of the 
  	underlying portfolio. The contingency is that the value of the portfolio 
  	must lie between a lower and an upper bound at expiration. If the value 
  	lies within these boundaries, the supershare is worth a proportion of the 
  	assets underlying the portfolio, else the supershare expires worthless. 
  	A supershare has a payoff that is basically like a spread of two 
  	asset-or-nothing calls, in which the owner of a supershare purchases an 
  	asset-or-nothing call with an strike price of the lower strike and sells 
  	an asset-or-nothing call with an strike price of the upper strike. The 
  	option price is calculated analytically according to Hakansson (1976).
  	\cr
  	[Haug's Book, Chapter 2.11.5]
  	\cr
  
  	
  	\bold{Binary Barrier Options:}
	\cr\cr
  	These options combine characteristics of both binary and barrier 
  	options. They are path dependent with a discontinuous payoff. Similar 
  	to barrier options, the payoff depends on whether or not the asset 
  	price crosses a predetermined barrier. There are 28 different types of 
  	binary barrier options, which can be divided into two main categories: 
  	Cash-or-nothing and Asset-or-nothing barrier options. Cash-or-nothing 
  	barrier options pay out a predetermined cash amount or nothing, depending 
  	on whether the asset price has hit the barrier. Asset-or-nothing barrier 
  	options pay out the value of the asset or nothing, depending on whether 
  	the asset price has crossed the barrier. The barrier monitoring frequency 
  	can be adjusted to account for discrete monitoring using an approximation 
  	developed by Broadie, Glasserman, and Kou (1995). Binary-barrier options 
  	can be priced analytically using a model introduced by Reiner and 
  	Rubinstein (1991).
  	\cr
  	[Haug's Book, Chapter 2.11.6]

}


\note{

  	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 2.11 of Haug's Book (1997).
  	
}


\value{

	The option price, a numeric value.
	
}


\examples{
## Examples from Chapter 2.11 in E.G. Haug's Option Guide (1997)

## Gap Option [2.11.1]:
   xmpOptions("\nStart: Gap Option > ")
   GapOption(TypeFlag  =  "c", S  =  50, X1  =  50, X2  =  57, Time  =  0.5, 
   	 r  =  0.09, b  =  0.09, sigma  =  0.20)

## Cash Or Nothing Option [2.11.2]:
   xmpOptions("\nNext: Cash or Nothing Option > ")
   CashOrNothingOption(TypeFlag  =  "p", S  =  100, X  =  80, K  =  10, 
   	 Time  =  9/12, r  =  0.06, b  =  0, sigma  =  0.35) 

## Two Asset Cash Or Nothing Option [2.11.3]:
   xmpOptions("\nNext: Two Asset Cash Or Nothing Option > ")
   # Type 1 - call:
   TwoAssetCashOrNothingOption(TypeFlag  =  "c", S1  =  100, S2  =  100, 
   	 X1  =  110, X2  =  90, K  =  10, Time  =  0.5, r  =  0.10, b1  =  0.05, 
   	 b2  =  0.06, sigma1  =  0.20, sigma2  =  0.25, rho  =  0.5)
   # Type 2 - put:
   TwoAssetCashOrNothingOption(TypeFlag  =  "p", S1  =  100, S2  =  100, 
   	 X1  =  110, X2  =  90, K  =  10, Time  =  0.5, r  =  0.10, b1  =  0.05, 
   	 b2  =  0.06, sigma1  =  0.20, sigma2  =  0.25, rho  =  -0.5)
   # Type 3 - down-up:
   TwoAssetCashOrNothingOption(TypeFlag  =  "ud", S1  =  100, S2  =  100, 
   	 X1  =  110, X2  =  90, K  =  10, Time  =  1, r  =  0.10, b1  =  0.05, 
   	 b2  =  0.06, sigma1  =  0.20, sigma2  =  0.25, rho  =  0)
   # Type 4 - up-down:
   TwoAssetCashOrNothingOption(TypeFlag  =  "du", S1  =  100, S2  =  100, 
   	 X1  =  110, X2  =  90, K  =  10, Time  =  1, r  =  0.10, b1  =  0.05, 
   	 b2  =  0.06, sigma1  =  0.20, sigma2  =  0.25, rho  =  0)

## Asset Or Nothing Option [2.11.4]: 
   xmpOptions("\nNext: Asset Or Nothing Option > ")
   AssetOrNothingOption(TypeFlag  =  "p", S  =  70, X  =  65, Time  =  0.5, 
     r  =  0.07, b  =  0.07 - 0.05, sigma  =  0.27)

## Super Share Option [2.11.5]:  
   xmpOptions("\nNext: Super Share Option > ")
   SuperShareOption(S  =  100, XL  =  90, XH  =  110, Time  =  0.25, r  =  0.10, 
     b  =  0, sigma  =  0.20)

## Binary Barrier Option [2.11.6]: 
   xmpOptions("\nNext: Binary Barrier Option > ")
   BinaryBarrierOption(TypeFlag = "6", S = 95, X=102, H = 100, 
     K = 15, Time = 0.5, r = 0.1, b = 0.1, sigma = 0.20)
   BinaryBarrierOption(TypeFlag = "12", S = 95, X = 98, H = 100, 
     K = 15, Time = 0.5, r = 0.1, b = 0.1, sigma = 0.20)
     
}


\references{
Cox J.C., Rubinstein M. (1985);
	\emph{Innovations in Option Markets},
	Prentice-Hall, New Jersey.

Hakkansson N.H. (1976);
	\emph{The Purchasing Power Fund: A New Kind of Financial
		Intermediary},
	Financial Analysts Journal 32, 49--59.

Haug E.G. (1997); 
	\emph{The complete Guide to Option Pricing Formulas}, 
	Chapter 2.11, McGraw-Hill, New York.
	
Heinen R.C., Kat H.M. (1996);
	\emph{Brick by Brick},
	Risk Magazine 9, 6.

Reiner E., Rubinstein M. (1991);
	\emph{Unscrambling the Binary Code};
	Risk Magazine 4, 9.
}


\author{

	Diethelm Wuertz for this R-Port.

}


\keyword{math}


\eof
\name{AsianOptions}

\alias{AsianOptions}

\alias{GeometricAverageRateOption}
\alias{TurnWakeAsianApproxOption}
\alias{LevyAsianApproxOption}


\title{Valuation of Asian Options}


\description{
  
	This is a collection of functions to valuate Asian options. 
	Asian options are path-dependent options, with payoffs that depend on 
  	the average price of the underlying asset or the average exercise 
  	price. There are two categories or types of Asian options: average 
  	rate options (also known as average price options) and average strike 
  	options. The payoffs depend on the average price of the underlying 
  	asset over a predetermined time period. An average is less volatile 
  	than the underlying asset, therefore making Asian options less 
  	expensive than standard European options. Asian options are commonly 
  	used in currency and commodity markets. Asian options are of interest 
  	in markets with thinly traded assets. Due to the little effect it 
  	will have on the options value, options based on an average, such 
  	as Asian options, have a reduced incentive to manipulate the 
  	underlying price at expiration.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab \code{GeometricAverageRateOption} 
	 		\tab Geometric Average Rate Option, \cr
	 	2 	\tab \code{TurnWakeAsianApproxOption}  
	 		\tab Turnbull and Wakeman's Approximation, \cr
	 	3 	\tab \code{LevyAsianApproxOption}      
	 		\tab Levy's Approximation. }
  	
}


\usage{
GeometricAverageRateOption(TypeFlag, S, X, Time, r, b, sigma)
TurnWakeAsianApproxOption(TypeFlag, S, SA, X, Time, time, tau, r, b, sigma)
LevyAsianApproxOption(TypeFlag, S, SA, X, Time, time, r, b, sigma)
}


\arguments{    

	\item{b}{
		the annualized cost-of-carry rate, a numeric value; 
		e.g. 0.1 means 10\% pa.
		}
	\item{r}{
		the annualized rate of interest, a numeric value; 
		e.g. 0.25 means 25\% pa.
		}
	\item{S, SA}{
		the asset price, a numeric value.
		}
	\item{sigma}{
		the annualized volatility of the underlying security, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.}
	\item{tau}{
	    [TurnWakeAsianApprox*] -
	  	is the time to the beginning of the average period.
	  	}
	\item{time, Time}{
		the time to maturity measured in years, a numeric value; 
		e.g. 0.5 means 6 months.
		}
	\item{TypeFlag}{
		a character string either \code{"c"} for a call option or 
		a \code{"p"} for a put option.
		}
	\item{X}{
		the exercise price, a numeric value.
		}
	
}


\details{

 	The Geometric average is the nth root of the product of the n sample 
  	points. The Arithmetic average is the sum of the stock values divided 
  	by the number of sampling points. Although Geometric Asian options are 
  	not commonly used in practice, they are often used as a good initial 
  	guess for the price of arithmetic Asian options. This technique is 
  	used to improve the convergence rate of the Monte Carlo model when 
  	pricing arithmetic Asian options.
  	\cr

  	Two cases are considered, the geometric and the arithmetic average-rate 
  	option. For the latter one can choose between three different kinds of 
  	approximations: Turnbull and Wakeman's approximations, Levy's approximation 
  	and Curran's approximation.
  	\cr
  	[Haug's Book, Chapter 2.12]
}

\note{

  	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 2.12 of Haug's Book (1997).
  	
}


\value{

	The option price, a numeric value.
	
}


\examples{
## Examples from Chapter 2.12 in E.G. Haug's Option Guide (1997)

## Geometric Average Rate Option:
   xmpOptions("\nStart: Geometric Average Rate Option > ")
   GeometricAverageRateOption(TypeFlag = "p", S = 80, X = 85, 
   	 Time = 0.25, r = 0.05, b = 0.08, sigma = 0.20)

## Turnbull Wakeman Approximation:
   xmpOptions("\nNext: Turnbull Wakeman Option > ")
   TurnWakeAsianApproxOption(TypeFlag = "p", S = 90, SA = 88, 
     X = 95, Time = 0.50, time = 0.25, tau = 0.0, r = 0.07, 
     b = 0.02, sigma = 0.25)

## Levy Asian Approximation:   
   xmpOptions("\nNext: Levy Asian Option > ")
   LevyAsianApproxOption(TypeFlag = "c", S = 100, SA = 100, X = 105, 
     Time = 0.75, time = 0.50, r = 0.10, b = 0.05, sigma = 0.15)
     
}


\references{
Haug E.G. (1997); 
	\emph{The complete Guide to Option Pricing Formulas}, 
	Chapter 2.12, McGraw-Hill, New York.
}


\author{

	Diethelm Wuertz for this R-Port.

}


\keyword{math}


\eof
\name{FXTransOptions}

\alias{FXTransOptions}

\alias{FEInDomesticFXOption}
\alias{QuantoOption}
\alias{EquityLinkedFXOption}
\alias{TakeoverFXOption}


\title{Valuation of Currency Translated Options}


\description{
  
	This is a collection of functions to valuate currency translated options.
  	Currency translated options are options on foreign assets where the 
  	payoff is exchanged into domestic currency at expiration. For example, 
  	a U.S. investor is interested in buying an option that is linked to 
  	the Nikkei index that is priced in yen. There are two types or risks, 
  	changing prices and exchange rates, to consider when valuing 
  	currency-translated options.
	\cr
	
	The functions are:

	\tabular{rll}{
		1 	\tab 	\code{FEInDomesticFXOption} 	
	 		\tab 	Foreign Exchange In Domestic Currency Option, \cr
		2 	\tab 	\code{QuantoOption} 				
	 		\tab 	Quanto Option, \cr
	 	3 	\tab 	\code{EquityLinkedFXOption} 		
	 		\tab 	Equity Linked FX Option, \cr
	 	4 	\tab 	\code{TakeoverFXOption} 			
	 		\tab 	Takeover FX Option. }
  	
}


\usage{
FEInDomesticFXOption(TypeFlag, S, E, X, Time, r, q, 
	sigmaS, sigmaE, rho)
QuantoOption(TypeFlag, S, Ep, X, Time, r, rf, q, sigmaS, 
	sigmaE, rho) 
EquityLinkedFXOption(TypeFlag, E, S, X, Time, r, rf, q, 
	sigmaS, sigmaE,rho)
TakeoverFXOption(V, B, E, X, Time, r, rf, sigmaV, sigmaE, rho)
}


\arguments{

	\item{B}{
  		[TakeoverFX*] - 
		the value of the foreign firm in the foreign currency at 
		the option expiration, a numeric value.
		}
	\item{E}{
  		[FEInDomesticFX*] - 
		the spot exchange rate specified in units of the domestic
		currency per unit of the foreign currency, a numeric value.\cr
		[TakeoverFX*] -
		the currency price quoted in units of the domestic currency
		per unit of the foreign currency.
		}
	\item{Ep}{
  		[Quanto*] -
  		the predetermined exchange rate specified in units of domestic
  		currency per unit of foreign currency.
  		}
  	\item{q}{
		[FEInDomesticFX*][EquityLinkedFX*] -
		the instantaneous proportional dividend payout rate of the 
		underlying asset, a numerical value.
		}	
	\item{r}{
		[FEInDomesticFX*][TakeoverFX*] -
		the domestic interest rate, a numeric value. E.g. 0.25 means 
		25\% p.a.
		}
	\item{rf}{
		[TakeoverFX*] -
		the foreign interest rate, a numeric value.
		}
	\item{rho}{
  		[TakeoverFX*] - 
		the correlation between annualized volatility of the currency price 
		quoted in units of the domestic currency per unit of the foreign 
		currency and the annualized volatility of the value of the foreign 
		firm, a numeric value.
		}	
	\item{S}{
  		[FEInDomesticFX*][EquityLinkedFX*] - 
		the underlying asset price in foreign currency, a numeric 
		value.
		}
	\item{sigmaE}{
  		[TakeoverFX*] - 
		the annualized volatility of the currency price quoted in 
		units of the domestic currency per unit of the foreign currency,
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}	
	\item{sigmaS}{
		[Quanto*] -
		the annualized volatility of the underlying asset,
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}
	\item{sigmaV}{
  		[TakeoverFX*] - 
		the annualized volatility of the value of the foreign firm, 
		a numeric value; e.g. 0.3 means 30\% volatility pa.
		}	
	\item{Time}{
		the time to maturity, a numeric value.
		}
	\item{TypeFlag}{
		a character string either "c" for a call option or a "p" 
		for a put option.
		}
	\item{V}{
  		[TakeoverFX*] - 
		the value of the foreign firm in the foreign currency, a 
		numeric value.
		}
	\item{X}{
		[FEInDomesticFX*] -
		the strike (delivery) price in domestic currency, a numeric 
		value. \cr
		[TakeoverFX*] -
		the strike price quoted in units of the domestic currency
		per unit of the foreign currency.
		}

}


\value{

	The option price, a numeric value.

}


\details{

	\bold{Equity Linked Foreign Exchange Options:}
  	\cr\cr
  	An equity-linked foreign-exchange option is an option on the foreign 
  	exchange rate and is linked to the forward price of a stock or equity 
  	index. This option can be priced analytically using a model introduced 
  	by Reiner (1992).
  	\cr
  	
  
  	\bold{Quanto Options:}
  	\cr\cr
  	A fixed exchange-rate foreign-equity option (Quanto) is denominated in 
  	another currency than that of the underlying equity exposure. The face 
  	value of the currency protection expands or contracts to cover changes 
  	in the foreign currency value of the underlying asset. Quanto options 
  	can be priced analytically using a model published by Dravid, Richardson, 
  	and Sun (1993).
  	\cr
  	
  	
  	\bold{Foreign Equity Options:}
  	\cr\cr
  	A foreign equity option is an option on a foreign asset where the strike 
  	price is specified in either domestic or foreign currency and the payoff 
  	at expiration is valued in domestic currency. Foreign equity options 
  	can be priced analytically using a model introduced by Reiner (1992).
  	\cr
  	
  
  	\bold{Takeover Foreign Exchange Options:}
  	\cr\cr
  	A takeover foreign exchange call option gives the buyer the right purchase 
  	a specified number of units of foreign currency at a strike price if the 
  	corporate takeover is successful. This option can be priced analytically 
  	using a model introduced by Schnabel and Wei (1994).
  	\cr
  	[Haug's Book, Chapter 2.13.4]

}


\note{

  	The functions implement the algorithms to valuate plain vanilla 
  	options as described in Chapter 1 of Haug's Book (1997).
  	
}


\examples{
## Examples from Chapter 2.13 in E.G. Haug's Option Guide (1997)

## Foreign Equity Options Struck in Domestic Currency [2.13.1]:
   xmpOptions("\nStart: FE in Domestic Currency Option > ")
   FEInDomesticFXOption(TypeFlag = "c", S = 100, E = 1.5, 
     X = 160, Time = 0.5, r = 0.08, q = 0.05, sigmaS = 0.20, 
     sigmaE = 0.12, rho = 0.45)

## Fixed Exchange-Rate Foreign-Equity Option [2.13.2]: 
   xmpOptions("\nNext: Quanto Option > ")
   QuantoOption(TypeFlag = "c", S = 100, Ep = 1.5, X = 105, 
     Time = 0.5, r = 0.08, rf = 0.05, q = 0.04, sigmaS= 0.2, 
     sigmaE = 0.10, rho = 0.30) 

## Equity Linked Foreign Exchange Option [2.13.3]:
   xmpOptions("\nNext: Equity Linked FX Option > ")
   EquityLinkedFXOption(TypeFlag = "p", E = 1.5, S = 100, 
     X = 1.52, Time = 0.25, r = 0.08, rf = 0.05, q = 0.04, 
     sigmaS = 0.20, sigmaE = 0.12, rho = -0.40)

## Takeover Foreign-Exchange Option [2.13.4]:
   xmpOptions("\nNext: Takeover FX Option > ")
   TakeoverFXOption(V = 100, B = 100, E = 1.5, X = 1.55, Time = 1, 
     r = 0.08, rf = 0.06, sigmaV = 0.20, sigmaE = 0.25, rho = 0.1)

}


\references{
Haug E.G. (1997); 
	\emph{The Complete Guide to Option Pricing Formulas}, 
	Chapter 2.13, McGraw-Hill, New York.
}


\author{

	Diethelm Wuertz for this R-Port.
	
}


\keyword{math}


\eof
\name{hngarchFit}

\alias{hngarchSim}
\alias{hngarchFit}
\alias{hngarchStats}
\alias{print.hngarch}
\alias{summary.hngarch}


\title{Heston-Nandi Garch(1,1) Modelling}


\description{

	This is a collection of functions to model the GARCH(1,1) price
	paths which underly Heston and Nandi's option pricing model.
	\cr
	
	The functions are:

	\tabular{rll}{	
	 	1 	\tab 	\code{hngarchSim}	
	 		\tab 	Simulates a Heston-Nandi Garch(1,1) process, \cr
	 	2 	\tab 	\code{hngarchFit}	
	 		\tab 	MLE for a Heston Nandi Garch(1,1) model, \cr
	 	3 	\tab 	\code{hngarchStats}	
	 		\tab 	True moments of the log-Return distribution, \cr
	 	4 	\tab 	\code{print.hngarch}	
	 		\tab 	Print method, \cr
	 	5 	\tab 	\code{summary.hngarch}	
	 		\tab 	Diagnostic summary.}
	
}


\usage{
hngarchSim(model, n, innov, n.start, start.innov, rand.gen, \dots)
hngarchFit(x, model, symmetric, trace, \dots)
hngarchStats(model)

\method{print}{hngarch}(x, \dots)
\method{summary}{hngarch}(object, \dots)
}


\arguments{

	\item{innov}{
		[hngarchSim] - \cr
		is a univariate time series or vector of innovations to produce 
		the series. If not provided, \code{innov} will be generated using 
		the random number generator specified by \code{rand.gen}. 
		Missing values are not allowed. By default the normal
		random number generator will be used.
		}
	\item{model}{
		a list of GARCH model parameters with the following entries:
		\code{lambda},
		\code{omega}, the constant coefficient of the variance equation, 
		\code{alpha} the autoregressive coefficient,
		\code{beta} the variance coefficient,
		\code{gamma} the asymmetry coefficient,
		and \code{rf}, the risk free rate, numeric values.	
		}
	\item{n}{
		[hngarchSim] - \cr
	    is the length of the series to be simulated. The default 
	    value is 1000.
	    }
	\item{n.start}{
		[hngarchSim] - \cr
	    gives the number of start-up values to be discarded.
	    The default value is 100.
	    } 
	\item{object}{
		[summary] - \cr
		a fitted HN-GARCH(1,1) time series object of class \code{"hngarch"} 
		as returned from the function \code{hngarchFit}.
		}
	\item{rand.gen}{
		[hngarchSim] - \cr
	    is the function which is called to generate the innovations. 
	    Usually, \code{rand.gen} will be a random number generator. 
	    Additional arguments required by the random number generator 
	    \code{rand.gen}, usually the location, scale and/or shape 
	    parameter of the underlying distribution function, have to be 
	    passed through the \code{dots} argument.
	    }
	\item{start.innov}{
		[hngarchSim] - \cr
	    is a univariate time series or vector of innovations to be used 
	    as start up values. Missing values are not allowed.
	    }
	\item{symmetric}{
		[hngarchFit] - \cr
		a logical, if TRUE a symmetric model is estimated, otherwise 
		the parameters are estimated for an asymmetric HN Garch(1,1) model.
		}
	\item{trace}{
		[hngarchFit] - \cr
		a logical value. Should the optimizarion be traced?
		If \code{trace=FALSE}, no tracing is done of the 
		iteration path.
		}
	\item{x}{
		[hngarchFit] - \cr
		an univariate vector or time series. \cr
		[print] - \cr
		a fitted HN-GARCH(1,1) time series object of class \code{"hngarch"} 
		as returned from the function \code{hngarchFit}.
		}
	\item{\dots}{
	    additional arguments to be passed.
	    }
	
}


\details
{

  	\bold{Path Simulation:}
  	\cr\cr
	The function \code{hngarchSim} simulates a Heston-Nandi Garch(1,1) 
	process with structure parameters specified through the list 
  	\code{model(lambda, omega, alpha, beta, gamma, rf)}.
    \cr

    
  	\bold{Parameter Estimation:}
  	\cr\ct
  	The function \code{hngarchFit} estimates by the maximum log-likelihood
  	approach the parameters either for a symmetric or an asymmetric 
  	Heston-Nandi Garch(1,1) model from the log returns \code{x} of a 
  	financial time series. For optimization R's \code{optim} function is
  	used. Additional optimization parameters may be passed throught the 
  	\code{\dots} argument.
  	\cr
  
  	
  	\bold{Diagnostic Analysis:}
  	\cr\cr
  	The function \code{summary.hngarch} performs a diagnostic analysis
  	and recalculates conditional variances and innovations from the time 
  	series.
  	\cr
  
  	
  	\bold{Calculation of Moments:}
  	\cr\cr
  	The function \code{hngarchStats} calculates the first four true 
  	moments of the unconditional log return distribution for a stationary 
  	Heston-Nandi Garch(1,1) process with standard normally distributed 
  	innovations. In addition the persistence and the expectation values 
  	of sigma to the power 2, 4, 6, and 8 can be accessed.
  	
}

\value{
  
	\code{hngarchSim}
	\cr
	returns numeric vector with the simulated time 
  	series points  neglecting those from the first \code{start.innov} 
  	innovations.  
  	\cr
  
  	\code{hngarchFit}
  	\cr
  	returns list with two entries: The estimated model parmeters 
  	\code{model}, where \code{model} is a list of the parameters 
  	itself, and \code{llh} the value of the log likelihood.
  	\cr
  	
  	\code{hngarchStats}
  	\cr
  	returns a list with the following components:
  	\code{persistence}, the value of the persistence,
  	\code{meansigma2}, \code{meansigma4}, \code{meansigma6}, \code{meansigma8}, 
  	the expectation value of sigma to the power of 2, 4, 6, and 8,
  	\code{mean}, \code{variance}, \code{skewness}, \code{kurtosis},
  	the mean, variance, skewness and kurtosis of the log returns.
  	\cr

  	\code{summary.hngarch}
  	\cr
  	returns list with the following elements: \code{h}, 
  	a numeric vector with the conditional variances, \code{z}, a numeric 
  	vector with the innovations.
	
}


\examples{
## hngarchSim -
## Simulate a Heston Nandi Garch(1,1) Process:
   xmpOptions("\nStart: Simulate a HN-GARCH(1,1) Process > ")
   # Symmetric Model - Parameters:
   model = list(lambda = 4, omega = 8e-5, alpha = 6e-5, 
     beta = 0.7, gamma = 0, rf = 0)
   ts = hngarchSim(model = model, n = 500, n.start = 100)
   par(mfrow = c(2, 1), cex = 0.75)
   ts.plot(ts, col = "steelblue4", main = "HN Garch Symmetric Model")
   grid()
   
## hngarchFit - 
## HN-GARCH log likelihood Parameter Estimation:
   xmpOptions("\nNext: Estimate a HN-GARCH(1,1) Process > ")
   # To speed up, we start with the simulated model ...
   mle = hngarchFit(model = model, x = ts, symmetric = TRUE)
   mle
   
## summary.hngarch - 
## HN-GARCH Diagnostic Analysis:
   xmpOptions("\nNext: Analyse a HN-GARCH(1,1) Process > ")
   par(mfrow = c(3, 1), cex = 0.75)
   summary(mle)    

## hngarchStats - 
## HN-GARCH Moments:
   xmpOptions("\nNext: Compute Moments of a HN-GARCH(1,1) Process > ")
   hngarchStats(mle$model)     
}


\references{
Heston S.L., Nandi S. (1997);
	\emph{A Closed-Form GARCH Option Pricing Model},
	Federal Reserve Bank of Atlanta.
}


\author{

	Diethelm Wuertz for this R-Port.
	
}


\keyword{models}


\eof
\name{HestonNandiOption}

\alias{HestonNandiOption}

\alias{HNGOption}
\alias{HNGGreeks}
\alias{HNGCharacteristics}


\title{Option Price for the Heston-Nandi Garch Option Model}


\description{
  
	This is a collection of functions to valuate Heston-Nandi options.
	Included are functions to compute the option price and the delta
	and gamma sensitivities for call and put options. 
	\cr
	
	The functions are:
	
	\tabular{rll}{
  		1 	\tab 	\code{HNGOption} 					
	 		\tab 	Heston-Nandi GARCH(1,1) option price, \cr
	    2 	\tab 	\code{HNGGreeks} 			
	 		\tab 	Heston-Nandi GARCH(1,1) option sensitivities, \cr
	  	3	\tab 	\code{HNGCharacteristics} 			
	 		\tab 	option prices and sensitivities. }
}


\usage{
HNGOption(TypeFlag, model, S, X, Time.inDays, r.daily)
HNGGreeks(Selection, TypeFlag, model, S, X, Time.inDays, r.daily)
HNGCharacteristics(TypeFlag, model, S, X, Time.inDays, r.daily)
}


\arguments{

  	\item{model}{
		a list of model parameters with the following entries:
		\code{lambda}, \code{omega}, \code{alpha}, \code{beta}, 
		and \code{gamma}, numeric values.
		}
  	\item{r.daily}{
		the daily rate of interest, a numeric value; 
		e.g. 0.25/252 means about 0.001\% per day.
		}
  	\item{S}{
		the asset price, a numeric value.
		}
  	\item{Selection}{
    	sensitivity to be computed, one of \code{"delta"}, \code{"gamma"},
    	\code{"vega"}, \code{"theta"}, \code{"rho"}, or \code{"CoC"}, 
    	a string value.
    	}
  	\item{Time.inDays}{
		the time to maturity measured in days, a numerical 
		value; e.g. 5/252 means 1 business week.
		}
  	\item{TypeFlag}{
		a character string either \code{"c"} for a call option or a 
		\code{"p"} for a put option.
		}
  	\item{X}{
		the exercise price, a numeric value.
		}
  
}


\value{

  	\code{HNGOption}
  	\cr
  	returns a list object of class \code{"option"} with \code{$price}
  	denoting the option price, a numeric value, and \code{$call} a 
  	character string which matches the function call.
  	\cr

  	
  	\code{HNGOGreeks} 
  	\cr
  	returns the option sensitivity for the selected Greek, either
  	\code{"delta"} or \code{"gamma"}; a numeric value.
  
  	
  	\code{HNGCharacteristics} 
  	\cr
  	returns a list with the following entries:
	
  	\item{premium}{
	  	the option price, a numeric value.}
	\item{delta}{
	  	the delta sensitivity, a numeric value.}
	\item{gamma}{
	  	the gamma sensitivity, a numeric value.}
	
}


\details{

  	\bold{Option Values:}
  	\cr\cr
  	\code{HNGOption}calculates the option price, \code{HNGGreeks}
  	allows to compute the option sensitivity Delta or Gamma, and 
  	\code{HNGcharacterisitics} summarizes both in one function call.
  	
}


\examples{
## model -
## Define the Model Parameters for a Heston-Nandi Option:
   xmpOptions("\nStart: Parameter Settings > ")
   model = list(lambda = -0.5, omega = 2.3e-6, alpha = 2.9e-6, 
   	 beta = 0.85, gamma = 184.25) 
   S = X = 100
   Time.inDays = 252
   r.daily = 0.05/Time.inDays
   sigma.daily = sqrt((model$omega + model$alpha) /
   	 (1 - model$beta - model$alpha * model$gamma^2))
   data.frame(S, X, r.daily, sigma.daily)

## HNGOption -
## Compute HNG Call-Put and compare with GBS Call-Put:
   xmpOptions("\nNext: HNG Option Prices > ")
   HNG = GBS = Diff = NULL
   for (TypeFlag in c("c", "p")) {
   	 HNG = c(HNG, HNGOption(TypeFlag, model = model, S = S, X = X, 
   	   Time.inDays = Time.inDays, r.daily = r.daily)$price )
     GBS = c(GBS, GBSOption(TypeFlag, S = S, X = X, Time = Time.inDays, 
       r = r.daily, b = r.daily, sigma = sigma.daily)$price) }
   Options = cbind(HNG, GBS, Diff = round(100*(HNG-GBS)/GBS, digits=2))
   row.names(Options) <- c("Call", "Put")
   data.frame(Options)
     
## HNGGreeks -
## Compute HNG Greeks and compare with GBS Greeks:
   xmpOptions("\nNext: HNG Delta and Gamma Sensitivities > ")
   Selection = c("Delta", "Gamma")
   HNG = GBS = NULL
   for (i in 1:2){
     HNG = c(HNG, HNGGreeks(Selection[i], TypeFlag = "c", model = model, 
       S = 100, X = 100, Time = Time.inDays, r = r.daily) ) 
     GBS = c(GBS, GBSGreeks(Selection[i], TypeFlag = "c", S = 100, X = 100, 
       Time = Time.inDays, r = r.daily, b = r.daily, sigma = sigma.daily) ) }
   Greeks = cbind(HNG, GBS, Diff = round(100*(HNG-GBS)/GBS, digits = 2))
   row.names(Greeks) <- Selection
   data.frame(Greeks)
}


\references{
Heston S.L., Nandi S. (1997);
	\emph{A Closed-Form GARCH Option Pricing Model},
	Federal Reserve Bank of Atlanta.
}


\author{

	Diethelm Wuertz for this R-port.
	
}


\keyword{math}


\eof
\name{LowDiscrepancy}

\alias{LowDiscrepancy}

\alias{runif.halton}
\alias{rnorm.halton}
\alias{runif.sobol}
\alias{rnorm.sobol}
\alias{runif.pseudo}
\alias{rnorm.pseudo}


\title{Low Discrepancy Sequences}


\description{
	
	This is a collection of functions to compute Halton's and
	Sobol's low discrepancy sequences, distributed in form of a
	uniform or normal distribution. 
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{runif.halton}		
	 		\tab	Uniform Halton sequence, \cr
	 	2 	\tab 	\code{rnorm.halton}		
	 		\tab	Normal Halton sequence, \cr
	 	3 	\tab 	\code{runif.sobol}		
	 		\tab	Uniform scrambled Sobol sequence, \cr
	 	4 	\tab 	\code{rnorm.sobol}		
	 		\tab	Normal scrambled Sobol sequence, \cr
	 	5 	\tab 	\code{runif.pseudo}		
	 		\tab	Uniform pseudo random numbers, \cr
	 	6 	\tab 	\code{norma.pseudo}		
	 		\tab	Normal pseudo random numbers.}
	 		
}


\usage{
runif.halton(n, dimension, init) 
rnorm.halton(n, dimension, init)

runif.sobol(n, dimension, init, scrambling, seed) 
rnorm.sobol(n, dimension, init, scrambling, seed)

runif.pseudo(n, dimension, init)
rnorm.pseudo(n, dimension, init)
}


\arguments{

  	\item{dimension}{
		an integer value, the dimension of the sequence. The 
		maximum value for the Sobol generator is 1111.
		}
  	\item{init}{
		a logical, if TRUE the sequence is initialized and
		restarts, otherwise not. By default TRUE.
		}
	\item{n}{
		an integer value, the number of random deviates.
		}
	\item{scrambling}{
		an integer value, if 1, 2 or 3 the sequence is scrambled
		otherwise not. If 1, Owen type type of scrambling is 
		applied, if 2, Faure-Tezuka type of scrambling, is
		applied, and if 3, both Owen+Faure-Tezuka type of
		scrambling is applied. By default 0.
		}
  	\item{seed}{
		an integer value, the random seed for initialization
		of the scrambling process. By default 4711. On effective
		if \code{scrambling>0}.
		}
		
}


\value{
  
	All generators return a numeric matrix of size \code{n} 
  	by \code{dimension}.
  	
}


\details{
	
	\bold{Halton's Low Discrepancy Sequences:}
	\cr\cr
	Calculates a matrix of uniform or normal deviated halton low 
	discrepancy numbers.
	\cr
	
	
	\bold{Scrambled Sobol's Low Discrepancy Sequences:}
	\cr\cr
	Calculates a matrix of uniform and normal deviated Sobol low 
	discrepancy numbers. Optional scrambling of the sequence can 
	be selected.
	\cr
	
	
	\bold{Pseudo Random Number Sequence:}
	\cr\cr
	Calculates a matrix of uniform or normal distributed pseudo 
	random numbers. This is a helpful function for comparing 
	investigations obtained from a low discrepancy series with 
	those from a pseudo random number.

}


\note{

	The global variables \code{runif.halton.seed} and 
	\code{runif.sobol.seed} save the status to restart the 
	generators. Note, that only one instance of a generators 
	can be run at the same time.
	
	The ACM Algorithm 659 implemented to generate scrambled 
	Sobol sequences is under the License of the ACM restricted 
	for academic and noncommerical usage. Please consult the 
	ACM License agreement included in the \code{doc} directory.
	
}


\author{
	
	P. Bratley and B.L. Fox for the Fortran Sobol Algorithm 659,\cr
	S. Joe for the Fortran extension to 1111 dimensions,\cr
	Diethelm Wuertz for this R-port.
	
}


\references{
Bratley P., Fox B.L. (1988);
	\emph{Algorithm 659: Implementing Sobol's Quasirandom
	Sequence Generator},
	ACM Transactions on Mathematical Software 14, 88--100.

Joe S., Kuo F.Y. (1998);
	\emph{Remark on Algorithm 659: Implementing Sobol's Quaisrandom
	Seqence Generator}.
}


\examples{
## *.halton - 
   xmpOptions("\nStart: Uniform Holton Numbers > ")
   par(mfrow = c(2, 2), cex = 0.75)
   runif.halton(n = 10, dimension = 5)
   hist(runif.halton(n = 5000, dimension = 1), main = "Uniform Halton", 
     xlab = "x", col = "steelblue3", border = "white")
   xmpOptions("\nNext: Normal Holton Numbers > ")
   rnorm.halton(n = 10, dimension = 5)
   hist(rnorm.halton(n = 5000, dimension = 1), main = "Normal Halton", 
     xlab = "x", col = "steelblue3", border = "white")
   
## *.sobol - 
   xmpOptions("\nNext: Uniform Sobol Numbers > ")
   runif.sobol(n = 10, dimension = 5, scrambling = 3)
   hist(runif.sobol(5000, 1, scrambling = 2), main = "Uniform Sobol", 
     xlab = "x", col = "steelblue3", border = "white")
   xmpOptions("\nNext: Normal Sobol Numbers > ")
   rnorm.sobol(n = 10, dimension = 5, scrambling = 3)
   hist(rnorm.sobol(5000, 1, scrambling = 2), main = "Normal Sobol", 
     xlab = "x", col = "steelblue3", border = "white")
   
## *.pseudo - 
   xmpOptions("\nNext: Uniform Pseudo Random Numbers > ")
   runif.pseudo(n = 10, dimension = 5)
   rnorm.pseudo(n = 10, dimension = 5) 
}


\keyword{programming}


\eof
\name{MonteCarloOptions}

\alias{MonteCarloOptions}

\alias{wienerMCPath}
\alias{plainVanillaMCPayoff}
\alias{arithmeticAsianMCPayoff}
\alias{MonteCarloOption}


\title{Monte Carlo Valuation of Options}


\description{

	This is a collection of functions to valuate options by Monte
	Carlo methods. The functions include beside the main Monte Carlo
	Simulator, example functions to generate Monte Carlo price paths 
	and to compute Monte Carlo price payoffs.
	\cr
	
	The functions are:

	\tabular{rll}{
	 	1 	\tab 	\code{sobolInnovations} 			
	 		\tab 	Example for scrambled Sobol innovations, \cr
	 	2 	\tab 	\code{wienerPath}  	
	 		\tab 	Example for a Wiener price path, \cr
	 	3 	\tab 	\code{plainVanillaPayoff}  
	 		\tab 	Example for the plain vanilla option's payoff, \cr
	 	4 	\tab 	\code{arithmeticAsianPayoff}  
	 		\tab 	Example for the arithmetic Asian option's payoff, \cr
	 	5 	\tab 	\code{MonteCarloOption} 		
	 		\tab 	Monte Carlo Simulator for options. }
	
}


\usage{
MonteCarloOption(delta.t, pathLength, mcSteps, mcLoops, init = TRUE, 
	innovations.gen, path.gen, payoff.calc, antithetic = TRUE, 
	standardization = FALSE, trace = TRUE, \dots)
}

\arguments{
  
	\item{antithetic}{
		a logical flag, should antithetic variates be used? 
		By default TRUE.
		}
	\item{delta.t}{
		the time step interval measured as a fraction of one
		year, by default one day, i.e. \code{delta.t=1/360}.
		}
	\item{init}{
		a logical flag, should the random number generator be 
		initialized? By default TRUE.
		}
	\item{innovations.gen}{
		a user defined function to generate the innovations, this can 
		be the normal random number generator \code{rnorm.pseudo} with 
		mean zero and variance one. For the usage of low discrepancy
		sequences alternativey \code{rnorm.halton} and \code{rnorm.sobol}
		can be called. The generator must deliver a normalized
		matrix of innovations with dimension given by the number of
		Monte Carlo steps and the path length. The first three
		arguments of the generator are the the number of Monte Carlo 
		steps \code{mcSteps}, the path length \code{pathLength} and the
		initialization flag \code{init}. Optional arguments can be passed
		through the argument \code{\dots}, e.g. the type of scrambling 
		for low discrepancy numbers.
		}
	\item{mcLoops, mcSteps}{
		the number of Monte Carlo loops and Monte Carlo Steps.
		In total \code{mcLoops*mcSteps} samples are included in
		one MC simulation.
		}
	\item{path.gen}{
		the user defined function to generate the price path.
		As the only input argument serves the matrix of innovations,
		the option parameters must be available as global variables.
		}
	\item{pathLength}{
		the length of the price path. This may be calculated
		as \code{floor(Time/delta.t)}, where \code{Time} denotes the 
		time to maturation measured in years.
		}
	\item{payoff.calc}{
		a user defined function to calculate the payoff of the option.
		As the only input argument serves the path matrix as returned
		by the path generator. The option parameters must be available 
		as global variables.
		}
	\item{standardization}{
		a logical flag, should the innovations for one loop be
		standardized? By default TRUE.
		}
	\item{trace}{
		a logical flag, should the Monte Carlo simulation be traced?
		By default TRUE.
		}
	\item{\dots}{
		additional arguments passed to the innovations generator.
		}	

}


\value{
  
	\emph{The user defined innovation generator}
	\cr
	returns a numeric matrix of (random) innovations to build the Monte 
	Carlo Paths.
	\cr
	
	\emph{The user defined path generator}
	\cr
	returns a numeric matrix of the Monte Carlo paths for the calculation
	of the option's payoffs.
	To be more precise, as an example the function returns for a Wiener
	process the matrix 
	\code{(b-sigma*sigma/2)*delta.t + sigma*sqrt(delta.t)*innovations}, 
	where the first term corresponds to the drift and the second to the 
	volatility.
	\cr
	
	
	\emph{The user defined payoff calculator},
	\cr
	returns the vector of the option's payoffs calculated from the generated
	paths. As an example this becomes for an arithmetic Asian call option
	with a Wiener Monte Carlo path \code{payoff = exp(-r*Time)*max(SM-X, 0)} 
	where code{SM = mean(S*exp(cumsum(path)))} and \code{path} denotes
	the MC price paths.
	\cr
	
	
	\bold{MonteCarloOption:}
	\cr	 
	returns a vector with the option prices for each Monte Carlo loop.
	
}


\details{
  	
 	\bold{The Innovations:}
	\cr\cr
	The innovations must created by the user defined innovation generator.
	The Generator has to return a numeric matrix of (random) innovations 
	of size \code{mcSteps} times the \code{pathLength}. The example
	section shows how to write sa function for scrambled Quasi Monte Carlo
	Sobol numbers. The package comes with three generators
	\code{rnorm.pseudo}, \code{rnorm.halton} and \code{rnorm.sobol}
	which can easily be used for simulations. 
	\cr
	
	\bold{The Price Paths:}
	\cr\cr
	The user must provide a function which generates the price paths.
	In the example section the function \code{wienerPath} creates a 
	Wiener Monte Carlo path from random innovations.
	The Wiener price path requires as input \code{b}, the annualized 
	cost-of-carry rate, and \code{sigma}, the annualized volatility of 
	the underlying security, to compute the drift and variance of the 
	path, these variables must be globally defined.
	\cr
	
	\bold{The Payoff Function:}
	\cr\cr
	The user must also provide a function which computes the payoff 
	value of the option. The example sections show how to write 
	payoff calculators for the plain vanilla option and for the 
	arithmetic Asian Option. As the only input argument the path matrix 
	is required. Again, the option parameters must be globally available.
	\cr
	
	\bold{The Monte Carlo Simulator:}
	\cr\cr	 
	The simulator is the heart of the Monte Carlo valuation process. 
	This simulator performs \code{mcLoops} Monte Carlo loops each with 
	\code{mcSteps} Monte Carlo steps. In each loop the following steps
	are done: first the innovation matrix is created from the specified
	innovation generator (usually build from the normal pseudo random 
	number or low discrepancy generators), then anththetic innovations 
	are added if desired (by default \code{anththetic=TRUE}), then the 
	innovations can be standardized within each loop (by default 
	\code{standardization=FALSE}), and finally the average payoff of 
	all samples in the loop is computed. The simulation can be traced 
	loop by loop setting the argument \code{trace=TRUE}.
	 
}


\author{

	Diethelm Wuertz for this R-Port.
	
}


\references{
Birge J.R. (1994);	
	\emph{Quasi-Monte Carlo Approaches to Option Pricing}, 
	Department of Industrial and Operations Engineering, 
	Technical Report 94--19, University of Michigan.

Boyle P. (1977);
	\emph{Options: A Monte Carlo approach},
	Journal of Finance, 32, 323--338.

Glasserman P. (2004); 
	\emph{Monte Carlo Methods in Financial Engineering},
	Springer-Verlag New York, Inc., 596 pp.

Jaeckel P. (2002); 
	\emph{Monte Carlo Methods in Finance}, 
	John Wiley and Sons Ltd, 222 pp. 

}


\examples{
## How to perform a Monte Carlo Simulation?
   
## First Step:
   xmpOptions("\nStart: Sobol Generator > ")
   # Write a function to generate the option's innovations. 
   # Use scrambled normal Sobol numbers:
   sobolInnovations = function(mcSteps, pathLength, init, ...) {
   	 # Create Normal Sobol Innovations:
   	 innovations = rnorm.sobol(mcSteps, pathLength, init, ...)
   	 # Return Value:
   	 innovations }
   	
## Second Step: 
   xmpOptions("\nNext: Wiener Path > ")
   # Write a function to generate the option's price paths.  
   # Use a Wiener path:
   wienerPath = function(eps) {	
	 # Note, the option parameters must be globally defined!
	 # Generate the Paths:
	 path = (b-sigma*sigma/2)*delta.t + sigma*sqrt(delta.t)*eps
	 # Return Value:
     path }
      
## Third Step: 
   # Write a function for the option's payoff
   
   # Example 1: use the payoff for a plain Vanilla Call or Put:
   xmpOptions("\nNext: Plain Vanilla Payoff > ")
   plainVanillaPayoff = function(path) { 
	 # Note, the option parameters must be globally defined!
	 # Compute the Call/Put Payoff Value:
	 ST = S*exp(sum(path))
	 if (TypeFlag == "c") payoff = exp(-r*Time)*max(ST-X, 0)
	 if (TypeFlag == "p") payoff = exp(-r*Time)*max(0, X-ST)
	 # Return Value:
	 payoff }
   
   # Example 2: use the payoff for an arithmetic Asian Call or Put:
   xmpOptions("\nNext: Arithmetic Asian Payoff > ")
   arithmeticAsianPayoff = function(path) { 
	 # Note, the option parameters must be globally defined!
	 # Compute the Call/Put Payoff Value:
	 SM = mean(S*exp(cumsum(path)))
	 if (TypeFlag == "c") payoff = exp(-r*Time)*max(SM-X, 0)
	 if (TypeFlag == "p") payoff = exp(-r*Time)*max(0, X-SM)
	 # Return Value:
	 payoff }

## Final Step: 
   xmpOptions("\nNext: Set Option Parameters > ")
   # Set Global Parameters for the plain Vanilla / arithmetic Asian Options:
   TypeFlag <<- "c"; S <<- 100; X <<- 100
   Time <<- 1/12; sigma <<- 0.4; r <<- 0.10; b <<- 0.1
   
   # Do the Asian Simulation with scrambled random numbers:
   xmpOptions("\nNext: Monte Carlo Simulation > ")
   mc = MonteCarloOption(delta.t = 1/360, pathLength = 30, mcSteps = 5000, 
     mcLoops = 50, init = TRUE, innovations.gen = sobolInnovations, 
     path.gen = wienerPath, payoff.calc = arithmeticAsianPayoff, 
     antithetic = TRUE, standardization = FALSE, trace = TRUE, 
     scrambling = 2, seed = 4711)
   
   # Plot the MC Iteration Path:
   xmpOptions("\nNext: Output Results > ")
   par(mfrow = c(1, 1))
   mcPrice = cumsum(mc)/(1:length(mc))
   plot(mcPrice, type = "l", main = "Arithmetic Asian Option", 
   	 xlab = "Monte Carlo Loops", ylab = "Option Price")
   
   # Compare with Turnbull-Wakeman Approximation:
   TW = TurnWakeAsianApproxOption(TypeFlag = "c", S = 100, SA = 100, 
   	 X = 100, Time = 1/12, time = 1/12, tau = 0 , r = 0.1, b = 0.1, 
   	 sigma = 0.4)$price
   print(TW)
   abline(h = TW, col = "red")
   grid()
 
}


\keyword{programming}


\eof
\name{fOptionsTools}


\alias{fOptionsTools}

\alias{xmpOptions}
\alias{xmpfOptions}


\title{fOptions Tools}


\description{

	Popups the example menu and starts or continues the example 
	and demo programs.
	
}


\usage{
xmpfOptions()
xmpOptions(prompt = "")
}


\arguments{

  	\item{prompt}{
		the string printed when prompting the user for input. 
		}	
}


\value{
  
	xmpfOptions
	\cr
	Popups the example menu.
	\cr
	
	xmpOptions
	\cr
	Nothing, the default, or the the prompt if you have set
	\code{xmpOptions = readline} on the command prompt. 
	
}


\details{

	The example in the manual pages may be interactive and ask for
	input from the user. To achieve this you have to type on the
	command line: \code{xmpOptions = readline}

}


\author{

	Diethelm Wuertz for this R-Port.

}


\examples{
## no examples ...
}


\keyword{programming}


\eof
